from __future__ import annotations

import locale
import logging
import warnings
import weakref

from semver.version import Version

from zospy.api import _ZOSAPI, constants
from zospy.api.apisupport import load_zosapi, load_zosapi_nethelper

logger = logging.getLogger(__name__)


class OpticStudioSystem:
    """Wrapper for OpticStudio System instances."""

    def __init__(self, zos_instance, system_instance):
        """Initiate the OpticStudioSystem.

        Parameters
        ----------
        zos_instance: ZOS
            A ZOS instance
        system_instance: ZOS.Application.PrimarySystem
            A PrimarySystem instance obtained from the zos_instance.
        """
        self._ZOS: ZOS = zos_instance

        self._System: _ZOSAPI.IOpticalSystem = system_instance
        self._OpenFile = None

    @property
    def SystemName(self) -> str:
        """Name of the current optical system."""
        return self._System.SystemName

    @SystemName.setter
    def SystemName(self, value: str):
        self._System.SystemName = value

    @property
    def SystemID(self) -> int:
        """Unique identifier for the optical system.

        This identifier can be used to differentiate between multiple `OpticStudioSystem` instances.

        Examples
        --------
        Create two different optical systems in a single instance of OpticStudio:

        >>> import zospy as zp
        >>> zos = zp.ZOS()
        >>> oss1 = zos.connect_as_standalone(return_primary_system=True)
        >>> oss2 = zos.create_new_system()
        >>> oss1.SystemID != oss2.SystemID
        True
        """
        return self._System.SystemID

    @property
    def Mode(self) -> str:
        """Mode of the optical system. Either "Sequential" or "NonSequential"."""
        return str(self._System.Mode)

    @property
    def SystemFile(self) -> str:
        """File path to the current optical system."""
        return self._System.SystemFile

    @property
    def IsNonAxial(self) -> bool:
        """Indicates whether the optical system is axial and sequential.

        `True` if the system is non-axial, `False` otherwise.
        """
        return self._System.IsNonAxial

    @property
    def NeedsSave(self) -> bool:
        """Indicates if the optical system contains unsaved changes."""
        return self._System.NeedsSave

    @property
    def SystemData(self) -> _ZOSAPI.SystemData.ISystemData:
        """Data for configuring everything in the system explorer.

        Examples
        --------
        Change the aperture type to "FloatByStopSize":

        >>> oss.SystemData.Aperture.ApertureType = zp.constants.SystemData.ZemaxApertureType.FloatByStopSize

        Add a wavelength of 543 nm with weight 1:

        >>> oss.SystemData.Wavelengths.AddWavelength(0.543, 1)
        """
        return self._System.SystemData

    @property
    def LDE(self) -> _ZOSAPI.Editors.LDE.ILensDataEditor:
        """Lens Data Editor."""
        return self._System.LDE

    @property
    def NCE(self) -> _ZOSAPI.Editors.NCE.INonSeqEditor:
        """Non-Sequential Component Editor."""
        return self._System.NCE

    @property
    def MFE(self) -> _ZOSAPI.Editors.MFE.IMeritFunctionEditor:
        """Merit Function Editor."""
        return self._System.MFE

    @property
    def TDE(self) -> _ZOSAPI.Editors.TDE.IToleranceDataEditor:
        """Tolerance Data Editor."""
        return self._System.TDE

    @property
    def MCE(self) -> _ZOSAPI.Editors.MCE.IMultiConfigEditor:
        """Multi-Configuration Editor."""
        return self._System.MCE

    @property
    def Analyses(self) -> _ZOSAPI.Analysis.I_Analyses:
        """Analyses for the current system."""
        return self._System.Analyses

    @property
    def Tools(self) -> _ZOSAPI.Tools.IOpticalSystemTools:
        """Interface to run various tools on the optical system."""
        return self._System.Tools

    @property
    def TheApplication(self) -> _ZOSAPI.IZOSAPI_Application:
        """Application in which the optical system is opened."""
        return self._System.TheApplication

    @property
    def LensUpdateMode(self) -> str:
        """Lens update mode of the optical system.

        Possible values are ['None', 'EditorsOnly', 'AllWindows'] or `zospy.constants.LensUpdateMode`.
        """
        return str(self._System.UpdateMode)

    @LensUpdateMode.setter
    def LensUpdateMode(self, value: constants.LensUpdateMode | str):
        self._System.UpdateMode = value

    @property
    def SessionModes(self) -> str:
        """Session mode of the optical system.

        Possible values are ['FromPreferences', 'SessionOn', 'SessionOff'], or `zospy.constants.SessionModes`.
        """
        return str(self._System.SessionMode)

    @SessionModes.setter
    def SessionModes(self, value: constants.SessionModes | str):
        self._System.SessionMode = constants.process_constant(constants.SessionModes, value)

    def get_current_status(self) -> str:
        """Get the last status of the optical system. If null or the length is 0, then the system has no errors.

        Returns
        -------
        str
            Current status of the optical system.
        """
        return self._System.GetCurrentStatus()

    def update_status(self) -> str:
        """Force an update of the system, and returns the current status.

        Returns
        -------
        str
            Current status of the optical system.
        """
        return self._System.UpdateStatus()

    def make_sequential(self) -> bool:
        """Set the optical system to sequential mode if it is not already."""
        return self._System.MakeSequential()

    def make_nonsequential(self) -> bool:
        """Set the optical system to non-sequential mode if it is not already."""
        return self._System.MakeNonSequential()

    def load(self, filepath: str, saveifneeded: bool = False):
        """Load an earlier defined Zemax file.

        Parameters
        ----------
        filepath: str
            The filepath that should be loaded
        saveifneeded: bool
            Defines if the current file is saved before opening the new file. Defaults to False.
        """
        logger.debug("Opening {} with SaveIfNeeded set to {}".format(filepath, saveifneeded))

        self._System.LoadFile(filepath, saveifneeded)
        self._OpenFile = filepath

        logger.info("Opened {}".format(filepath))

    def new(self, saveifneeded: bool = False):
        """Create a new session file.

        Parameters
        ----------
        saveifneeded: bool
            Defines if the current file is saved before opening the new file. Defaults to False.
        """
        logger.debug("Creating new file")

        self._System.New(saveifneeded)
        self._OpenFile = None

        logger.info("Opened new file")

    def save_as(self, filepath: str):
        """Save the current session under a specified name.

        Parameters
        ----------
        filepath: str
            The filepath where the system should be saved. Note that a partial path or relative path does not work.
        """
        logger.debug("Saving open session as {}".format(filepath))

        self._System.SaveAs(filepath)
        self._OpenFile = filepath

        logger.info("File saved as {}".format(filepath))

    def save(self) -> bool:
        """Save the current OpticStudio session.

        If the file name for the current session is not known (e.g. when a new file was created), a warning is raised
        and the file is not saved. On these occurences, save_as() should be used once.

        Returns (bool): A boolean indicating if the saving was successful.

        """
        logger.debug("Saving file")

        if self._OpenFile is None:
            warnings.warn(
                "No file name has been defined for the current system, the current session has not been "
                "saved. Please use the save_as() function before using save."
            )
            logger.critical("Could not save file")

            return False
        else:
            self._System.Save()
            logger.info("File saved")
            return True

    def close(self, saveifneeded: bool = False) -> bool:
        """Close the optical system. Only works on secondary systems (See OpticStudio documentation).

        Parameters
        ----------
        saveifneeded : bool
            Defines if the current file is saved before opening the new file. Defaults to False.
        """
        return self._System.Close(saveifneeded)

    def copy_system(self) -> OpticStudioSystem:
        """Copy the current OpticStudioSystem instance.

        Returns
        -------
        zospy.core.OpticStudioSystem
            A ZOSPy OpticStudioSystem instance. Should be sequential.
        """
        return OpticStudioSystem(self._ZOS, self._System.CopySystem())

    def _ensure_correct_mode(self, required: str):
        """Ensure that the system is in the required type.

        Parameters
        ----------
        required: str
            The required system type. Either 'Sequential' or 'NonSequential'.

        Returns
        -------
        None

        Raises
        ------
        ValueError:
            When the system is in the incorrect type
        """
        logger.debug("Ensuring correct mode")
        if self.Mode != required:
            err = "Incorrect system type. System is in {} mode but {} mode is required".format(self.Mode, required)
            logger.critical(err)
            raise TypeError(err)
        else:
            logger.debug("System is in correct mode")
        # ToDo: Eveluate what happens in 'mixed mode'

    def __del__(self):
        logger.debug("Closing connections with Zemax OpticStudio")
        # ToDo Add cleanup


class ZOS:
    """A Communication instance for Zemax OpticStudio.

    This class can be used to establish a link between Python and Zemax OpticStudio through .NET,
    and subsequently control OpticStudio. Only one instance of `ZOS` can exist at any time.

    The connection is established in two ways, the preferred method as wel as a legacy method for backwards
    compatability. See examples.

    Parameters
    ----------
    preload : bool
        A boolean indicating if nested namespaces should be preloaded upon initiating ZOS. Defaults to False.
    zosapi_nethelper : str | None
        Optional filepath to the ZOSAPI_NetHelper dll that is required to connect to OpticStudio. If None, the
        Windows registry will be used to find the ZOSAPI_NetHelper dll. Defaults to None.

    Attributes
    ----------
    ZOSAPI : None | netModuleObject
        The ZOSAPI interface once loaded, else None.
    ZOSAPI_NetHelper : None | netModuleObject
        The ZOSAPI_NetHelper interface once loaded, else None.

    Raises
    ------
    ValueError
        When it is attempted to initiate a second instance of  `ZOS`. Only one instance can exist at any time.

    Examples
    --------
    Preferred methods:

    1. Connecting as extension:

    >>> import zospy as zp
    >>> zos = zp.ZOS()
    >>> oss = zos.connect_as_extension(return_primary_system=True)

    2. Launching OpticStudio in standalone mode:

    >>> import zospy as zp
    >>> zos = zp.ZOS()
    >>> oss = zos.create_new_application(return_primary_system=True)

    Legacy methods:

    1. Connecting as extension:

    >>> import zospy as zp
    >>> zos = zp.ZOS()
    >>> zos.wakeup()
    >>> zos.connect_as_extension()
    >>> oss = zos.get_primary_system()

    2. Launching OpticStudio in standalone mode:

    >>> import zospy as zp
    >>> zos = zp.ZOS()
    >>> zos.wakeup()
    >>> zos.create_new_application()
    >>> oss = zos.get_primary_system()
    """

    _OpticStudioSystem = OpticStudioSystem

    _instances = set()

    def __new__(cls, *args, **kwargs):
        if len(cls._instances) >= 1:
            # As the number of applications within runtime is limited to 1 by Zemax, it is logical to also limit the
            # number of ZOS instances
            raise ValueError(
                "Cannot have more than one active ZOS instance.\n\n"
                "Since OpticStudio limits the number of connections to the ZOS-API to 1, only a single ZOS instance "
                "is allowed. Re-use the existing instance, or delete the existing instance prior to initializing a "
                "new one."
            )

        instance = super(ZOS, cls).__new__(cls)

        return instance

    def __init__(self, preload: bool = False, zosapi_nethelper: str = None):
        """Initiation of the ZOS instance.

        The ZOS instance can subsequently be used to connect to OpticStudio. See the examples in the class docstring for
        more information.

        Parameters
        ----------
        preload : bool
            A boolean indicating if nested namespaces should be preloaded upon initiating ZOS. Defaults to False.
        zosapi_nethelper : str | None
            Optional filepath to the ZOSAPI_NetHelper dll that is required to connect to OpticStudio. If None, the
            Windows registry will be used to find the ZOSAPI_NetHelper dll. Defaults to None.
        """
        logger.debug("Initializing ZOS instance")

        self.ZOSAPI: _ZOSAPI = None
        self.ZOSAPI_NetHelper = None
        self.Connection: _ZOSAPI.IZOSAPI_Connection = None
        self.Application: _ZOSAPI.IZOSAPI_Application = None

        # Register the instance and create a finalize code to remove it from ZOS._instances when deleted
        ZOS._instances.add(id(self))
        weakref.finalize(self, ZOS._instances.discard, id(self))

        logger.info("ZOS instance initialized")

        self.wakeup(preload=preload, zosapi_nethelper=zosapi_nethelper)

    def wakeup(self, preload: bool = False, zosapi_nethelper: str = None):
        """Wake the zosapi instance.

        .. deprecated:: 1.1.0
                `wakeup` will be removed in ZOSPy 2.0.0, as it is automatically called by `__init__`.

        The parameters are passed to self._load_zos_dlls().

        Parameters
        ----------
        preload : bool
            A boolean indicating if nested namespaces should be preloaded.
        zosapi_nethelper : str, optional
            File path to the ZOSAPI_NetHelper dll, if None, the Windows registry will be used to find
            ZOSAPI_NetHelper dll. Defaults to None.

        Returns
        -------
        None
        """
        if self.Connection is None:
            self._load_zos_dlls(preload=preload, zosapi_nethelper=zosapi_nethelper)
            self._assign_connection()

    def _load_zos_dlls(self, preload: bool = False, zosapi_nethelper: str = None):
        """Load the ZOS-API DLLs and makes them available for usage through ZOS.ZOSAPI and ZOS.ZOSAPI_NetHelper.

        As there are many nested namespaces in the ZOSAPI DLLs that are always available after import but not
        preloaded, it can be chosen to do so. This should only be done for developmental reasons, as it likely slows
        down the code.

        Parameters
        ----------
        preload: bool
            A boolean indicating if nested namespaces should be preloaded.
        zosapi_nethelper: str, optional
            Optional filepath to the ZOSAPI_NetHelper dll, if None, the windows registry will be used to find
            ZOSAPI_NetHelper dll. Defaults to None.

        Raises
        ------
        FileNotFoundError:
            An error occurred in locating one of the DLLs.
        ModuleNotFoundError:
            One of the nested namespaces cannot be found. This error can only occur with preload set to True.
        """
        logger.debug("Loading ZOS DLLs with preload set to {}".format(preload))
        self.ZOSAPI_NetHelper = load_zosapi_nethelper(filepath=zosapi_nethelper, preload=preload)
        self.ZOSAPI = load_zosapi(self.ZOSAPI_NetHelper, preload=preload)

    def _assign_connection(self):
        """Assign the ZOSAPI Connection to self.Connection."""
        if not self.Connection:
            logger.debug("Assigning ZOSAPI_Connection() to self.Connection")
            self.Connection = self.ZOSAPI.ZOSAPI_Connection()
        else:
            logger.debug("ZOSAPI_Connection() already assigned self.Connection")

    def connect_as_extension(
        self, instancenumber: int = 0, return_primary_system: bool = False
    ) -> bool | OpticStudioSystem:
        """Connect to Zemax OpticStudio as extension.

        The application will be assigned to ZOS.Application.

        Parameters
        ----------
        instancenumber : int, optional
            An integer to specify the number of the instance used.
        return_primary_system: bool, optional
            A boolean indicating if the primary OpticStudioSystem should be returned. Defaults to `False`.

        Returns
        -------
        bool | OpticStudioSystem
            `True` if a valid connection is made, else `False`. If `return_primary_system` is `True`, the function
            returns the primary `OpticStudioSystem`.
        """
        self._assign_connection()

        if self.Connection.IsAlive:  # ToDo ensure no memory leak
            raise RuntimeError("Only one Zemax application can exist within runtime")

        self.Application = self.Connection.ConnectAsExtension(instancenumber)

        if not self.Application.IsValidLicenseForAPI:
            logger.critical("OpticStudio Licence is not valid for API, connection not established")

            if return_primary_system:
                raise ConnectionRefusedError("OpticStudio Licence is not valid for API, connection not established")

            return False

        if return_primary_system:
            return self.get_primary_system()

        return True

    def create_new_application(self, return_primary_system: bool = False) -> bool | OpticStudioSystem:
        """Create a standalone Zemax OpticStudio instance.

        The application will be assigned to ZOS.Application.

        Parameters
        ----------
        return_primary_system : bool, optional
            A boolean indicating if the primary OpticStudioSystem should be returned. Defaults to `False`.

        Returns
        -------
        bool | OpticStudioSystem
            `True` if a valid connection is made, else `False`. If `return_primary_system` is `True`, the function
            returns the primary `OpticStudioSystem`.
        """
        self._assign_connection()

        if self.Connection.IsAlive:  # ToDo ensure no memory leak
            raise RuntimeError("Only one Zemax application can exist within runtime")

        self.Application = self.Connection.CreateNewApplication()

        if not self.Application.IsValidLicenseForAPI:
            logger.critical("OpticStudio Licence is not valid for API, connection not established")

            if return_primary_system:
                raise ConnectionRefusedError("OpticStudio Licence is not valid for API, connection not established")

            return False

        if return_primary_system:
            return self.get_primary_system()

        return True

    def connect_as_standalone(self, return_primary_system: bool = False) -> bool | OpticStudioSystem:
        """Creates a standalone Zemax OpticStudio instance.

        Equal to `ZOS.create_new_application`.

        Parameters
        ----------
        return_primary_system : bool, optional
            A boolean indicating if the primary OpticStudioSystem should be returned. Defaults to `False`.

        Returns
        -------
        bool | OpticStudioSystem
            `True` if a valid connection is made, else `False`. If `return_primary_system` is `True`, the function
            returns the primary `OpticStudioSystem`.
            runs ZOS.get_primary_system() and directly returns OpticStudioSystem.
        """
        return self.create_new_application(return_primary_system=return_primary_system)

    def create_new_system(self, system_mode: constants.SystemType | str = "Sequential") -> OpticStudioSystem:
        """Creates a new OpticStudioSystem. This works only if ZOSPy is connected to a standalone application.

        Parameters
        ----------
        system_mode : constants.SystemType | str
            The mode of the new system, must be one of "Sequential", "NonSequential". Defaults to "Sequential".

        Returns
        -------
        OpticStudioSystem
            The new OpticStudioSystem.
        """
        if self.Application.Mode == constants.ZOSAPI_Mode.Server:
            new_system = self.Application.CreateNewSystem(constants.process_constant(constants.SystemType, system_mode))
            return self._OpticStudioSystem(zos_instance=self, system_instance=new_system)

        raise ValueError("Can only create a new system when using a standalone connection.")

    def get_primary_system(self) -> OpticStudioSystem:
        """
        Get the primary optical system.

        Returns
        -------
        OpticStudioSystem
            Primary optical system.
        """
        opticstudiosystem = self.Application.PrimarySystem
        return self._OpticStudioSystem(zos_instance=self, system_instance=opticstudiosystem)

    def get_system(self, pos: int = 0) -> OpticStudioSystem:
        """
        Get the optical system at the specified position.

        Parameters
        ----------
        pos : int
            Index of the optical system. If `0`, the primary system is returned.

        Returns
        -------
        OpticStudioSystem
            Optical system at position `pos`.
        """
        opticstudiosystem = self.Application.GetSystemAt(pos)
        return self._OpticStudioSystem(zos_instance=self, system_instance=opticstudiosystem)

    def get_txtfile_encoding(self) -> str:
        """Determines the encoding used to write textfiles in OpticStudio.

        Returns
        -------
        str
            The encoding used for textfiles by OpticStudio

        Raises
        ------
        RuntimeError
            When ZOS does not have a connection to the OpticStudio application
        NotImplementedError
            When the ZOS.Application.Preferences.General.TXTFileEncoding is not one of ["Unicode", "ANSI"]
        """
        if self.Application is None:
            raise RuntimeError("ZOS.get_txtfile_encoding requires a live connection to the OpticStudio application.")

        if str(self.Application.Preferences.General.TXTFileEncoding) == "Unicode":
            return "UTF-16-le"
        elif str(self.Application.Preferences.General.TXTFileEncoding) == "ANSI":
            return locale.getpreferredencoding(do_setlocale=False)
        else:
            raise NotImplementedError(
                f"ZOSPy cannot handle encoding {str(self.Application.Preferences.General.TXTFileEncoding)}"
            )

    @property
    def version(self) -> Version:
        """Returns the OpticStudio version as Version object."""
        return Version(
            major=self.Application.ZOSMajorVersion,
            minor=self.Application.ZOSMinorVersion,
            patch=self.Application.ZOSSPVersion,
        )



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Analysis import SampleSizes, SampleSizes_Pow2Plus1
from zospy.api._ZOSAPI.Analysis.Settings import (
    IAS_,
    IAS_Field,
    IAS_Surface,
    IAS_Wavelength,
)

__all__ = ("IAS_DiffEfficiency2D", "IAS_DiffEfficiencyAngular", "IAS_DiffEfficiencyChromatic")

class IAS_DiffEfficiency2D(IAS_):
    @property
    def AmbientObject(self) -> int: ...
    @property
    def DetectorObject(self) -> int: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def HologramObject(self) -> int: ...
    @property
    def InsideOfObject(self) -> int: ...
    @property
    def MaxAngle(self) -> float: ...
    @property
    def MaxWavelength(self) -> float: ...
    @property
    def MinAngle(self) -> float: ...
    @property
    def MinWavelength(self) -> float: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def srcPolJx(self) -> float: ...
    @property
    def srcPolJy(self) -> float: ...
    @property
    def srcPositionX(self) -> float: ...
    @property
    def srcPositionY(self) -> float: ...
    @property
    def srcPositionZ(self) -> float: ...
    @property
    def srcPrePropagation(self) -> float: ...
    @property
    def srcUnpolarized(self) -> bool: ...
    @property
    def srcXTitled(self) -> bool: ...
    @property
    def Surface(self) -> IAS_Surface: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @AmbientObject.setter
    def AmbientObject(self, value: int) -> None: ...
    @DetectorObject.setter
    def DetectorObject(self, value: int) -> None: ...
    @HologramObject.setter
    def HologramObject(self, value: int) -> None: ...
    @InsideOfObject.setter
    def InsideOfObject(self, value: int) -> None: ...
    @MaxAngle.setter
    def MaxAngle(self, value: float) -> None: ...
    @MaxWavelength.setter
    def MaxWavelength(self, value: float) -> None: ...
    @MinAngle.setter
    def MinAngle(self, value: float) -> None: ...
    @MinWavelength.setter
    def MinWavelength(self, value: float) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @srcPolJx.setter
    def srcPolJx(self, value: float) -> None: ...
    @srcPolJy.setter
    def srcPolJy(self, value: float) -> None: ...
    @srcPositionX.setter
    def srcPositionX(self, value: float) -> None: ...
    @srcPositionY.setter
    def srcPositionY(self, value: float) -> None: ...
    @srcPositionZ.setter
    def srcPositionZ(self, value: float) -> None: ...
    @srcPrePropagation.setter
    def srcPrePropagation(self, value: float) -> None: ...
    @srcUnpolarized.setter
    def srcUnpolarized(self, value: bool) -> None: ...
    @srcXTitled.setter
    def srcXTitled(self, value: bool) -> None: ...

class IAS_DiffEfficiencyAngular(IAS_):
    @property
    def AmbientObject(self) -> int: ...
    @property
    def DetectorObject(self) -> int: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def HologramObject(self) -> int: ...
    @property
    def InsideOfObject(self) -> int: ...
    @property
    def MaxAngle(self) -> float: ...
    @property
    def MinAngle(self) -> float: ...
    @property
    def SampleSize(self) -> SampleSizes_Pow2Plus1: ...
    @property
    def srcPolJx(self) -> float: ...
    @property
    def srcPolJy(self) -> float: ...
    @property
    def srcPositionX(self) -> float: ...
    @property
    def srcPositionY(self) -> float: ...
    @property
    def srcPositionZ(self) -> float: ...
    @property
    def srcPrePropagation(self) -> float: ...
    @property
    def srcUnpolarized(self) -> bool: ...
    @property
    def srcXTitled(self) -> bool: ...
    @property
    def Surface(self) -> IAS_Surface: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @AmbientObject.setter
    def AmbientObject(self, value: int) -> None: ...
    @DetectorObject.setter
    def DetectorObject(self, value: int) -> None: ...
    @HologramObject.setter
    def HologramObject(self, value: int) -> None: ...
    @InsideOfObject.setter
    def InsideOfObject(self, value: int) -> None: ...
    @MaxAngle.setter
    def MaxAngle(self, value: float) -> None: ...
    @MinAngle.setter
    def MinAngle(self, value: float) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes_Pow2Plus1) -> None: ...
    @srcPolJx.setter
    def srcPolJx(self, value: float) -> None: ...
    @srcPolJy.setter
    def srcPolJy(self, value: float) -> None: ...
    @srcPositionX.setter
    def srcPositionX(self, value: float) -> None: ...
    @srcPositionY.setter
    def srcPositionY(self, value: float) -> None: ...
    @srcPositionZ.setter
    def srcPositionZ(self, value: float) -> None: ...
    @srcPrePropagation.setter
    def srcPrePropagation(self, value: float) -> None: ...
    @srcUnpolarized.setter
    def srcUnpolarized(self, value: bool) -> None: ...
    @srcXTitled.setter
    def srcXTitled(self, value: bool) -> None: ...

class IAS_DiffEfficiencyChromatic(IAS_):
    @property
    def AmbientObject(self) -> int: ...
    @property
    def DetectorObject(self) -> int: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def HologramObject(self) -> int: ...
    @property
    def InsideOfObject(self) -> int: ...
    @property
    def MaxWavelength(self) -> float: ...
    @property
    def MinWavelength(self) -> float: ...
    @property
    def SampleSize(self) -> SampleSizes_Pow2Plus1: ...
    @property
    def srcPolJx(self) -> float: ...
    @property
    def srcPolJy(self) -> float: ...
    @property
    def srcPositionX(self) -> float: ...
    @property
    def srcPositionY(self) -> float: ...
    @property
    def srcPositionZ(self) -> float: ...
    @property
    def srcPrePropagation(self) -> float: ...
    @property
    def srcUnpolarized(self) -> bool: ...
    @property
    def srcXTitled(self) -> bool: ...
    @property
    def Surface(self) -> IAS_Surface: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @AmbientObject.setter
    def AmbientObject(self, value: int) -> None: ...
    @DetectorObject.setter
    def DetectorObject(self, value: int) -> None: ...
    @HologramObject.setter
    def HologramObject(self, value: int) -> None: ...
    @InsideOfObject.setter
    def InsideOfObject(self, value: int) -> None: ...
    @MaxWavelength.setter
    def MaxWavelength(self, value: float) -> None: ...
    @MinWavelength.setter
    def MinWavelength(self, value: float) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes_Pow2Plus1) -> None: ...
    @srcPolJx.setter
    def srcPolJx(self, value: float) -> None: ...
    @srcPolJy.setter
    def srcPolJy(self, value: float) -> None: ...
    @srcPositionX.setter
    def srcPositionX(self, value: float) -> None: ...
    @srcPositionY.setter
    def srcPositionY(self, value: float) -> None: ...
    @srcPositionZ.setter
    def srcPositionZ(self, value: float) -> None: ...
    @srcPrePropagation.setter
    def srcPrePropagation(self, value: float) -> None: ...
    @srcUnpolarized.setter
    def srcUnpolarized(self, value: bool) -> None: ...
    @srcXTitled.setter
    def srcXTitled(self, value: bool) -> None: ...



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

from . import (
    RMS,
    Aberrations,
    EncircledEnergy,
    ExtendedScene,
    Fans,
    Mtf,
    NSCSurface,
    Psf,
    Spot,
    Wavefront,
)

__all__ = (
    "Aberrations",
    "EncircledEnergy",
    "ExtendedScene",
    "Fans",
    "Mtf",
    "NSCSurface",
    "Psf",
    "RMS",
    "Spot",
    "Wavefront",
    "AxisType",
    "DetectorViewerScaleTypes",
    "DisplayOption",
    "HuygensPsfTypes",
    "Parity",
    "Polarizations",
    "PsfSpread",
    "PsfTypes",
    "ReferenceGia",
    "Rotations",
    "ScanTypes",
    "SourceGia",
    "STAREffectsOptions",
)

class AxisType(Enum):
    X = 0
    Y = 1
    Z = 2

class DetectorViewerScaleTypes(Enum):
    Linear = 0
    Log_Minus_5 = 1
    Normalized = 1
    Log_Minus_10 = 2
    Log_Minus_15 = 3

class DisplayOption(Enum):
    AllRays = 0
    FailedRays = 1
    PassedRays = 2

class HuygensPsfTypes(Enum):
    Linear = 0
    Log_Minus_1 = 1
    Log_Minus_2 = 2
    Log_Minus_3 = 3
    Log_Minus_4 = 4
    Log_Minus_5 = 5
    Real = 6
    Imaginary = 7
    Phase = 8

class Parity(Enum):
    Even = 0
    Odd = 1

class Polarizations(Enum):
    None_ = 0
    Ex = 1
    Ey = 2
    Ez = 3

class PsfSpread(Enum):
    Line = 0
    Edge = 1

class PsfTypes(Enum):
    X_Linear = 0
    Y_Linear = 1
    X_Logarithmic = 2
    Y_Logarithmic = 3
    X_Phase = 4
    Y_Phase = 5
    X_RealPart = 6
    Y_RealPart = 7
    X_ImaginaryPart = 8
    Y_ImaginaryPart = 9

class ReferenceGia(Enum):
    ChiefRay = 0
    Vertex = 1
    PrimaryChief = 2
    Centroid = 3

class Rotations(Enum):
    Rotate_0 = 0
    Rotate_90 = 1
    Rotate_180 = 2
    Rotate_270 = 3

class ScanTypes(Enum):
    Plus_Y = 0
    Plus_X = 1
    Minus_Y = 2
    Minus_X = 3

class SourceGia(Enum):
    Uniform = 0
    Lambertian = 1

class STAREffectsOptions(Enum):
    On = 0
    Difference = 1



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

from . import PhysicalOptics, RayTracing, Settings, Tolerancing

__all__ = (
    "PhysicalOptics",
    "RayTracing",
    "Settings",
    "Tolerancing",
    "AnalysisIDM",
    "Beam",
    "BestFitSphereOptions",
    "ColorPaletteType",
    "DetectorViewerShowAsTypes",
    "DetectorViewerShowDataTypes",
    "ErrorType",
    "GiaShowAsTypes",
    "GridPlotType",
    "HuygensShowAsTypes",
    "HuygensSurfaceMftShowAsTypes",
    "POPSampling",
    "RemoveOptions",
    "SampleSizes",
    "SampleSizes_ContrastLoss",
    "SampleSizes_Pow2Plus1",
    "SampleSizes_Pow2Plus1_X",
    "ShowAs",
    "SurfaceCurvatureCrossData",
    "SurfaceCurvatureData",
    "SurfacePhaseData",
    "SurfaceSagData",
    "SurfaceSlopeCrossData",
    "SurfaceSlopeData",
    "UserAnalysisDataType",
)

class AnalysisIDM(Enum):
    RayFan = 0
    OpticalPathFan = 1
    PupilAberrationFan = 2
    FieldCurvatureAndDistortion = 3
    FocalShiftDiagram = 4
    GridDistortion = 5
    LateralColor = 6
    LongitudinalAberration = 7
    RayTrace = 8
    SeidelCoefficients = 9
    SeidelDiagram = 10
    ZernikeAnnularCoefficients = 11
    ZernikeCoefficientsVsField = 12
    ZernikeFringeCoefficients = 13
    ZernikeStandardCoefficients = 14
    FftMtf = 15
    FftThroughFocusMtf = 16
    GeometricThroughFocusMtf = 17
    GeometricMtf = 18
    FftMtfMap = 19
    GeometricMtfMap = 20
    FftSurfaceMtf = 21
    FftMtfvsField = 22
    GeometricMtfvsField = 23
    HuygensMtfvsField = 24
    HuygensMtf = 25
    HuygensSurfaceMtf = 26
    HuygensThroughFocusMtf = 27
    FftPsf = 28
    FftPsfCrossSection = 29
    FftPsfLineEdgeSpread = 30
    HuygensPsfCrossSection = 31
    HuygensPsf = 32
    DiffractionEncircledEnergy = 33
    GeometricEncircledEnergy = 34
    GeometricLineEdgeSpread = 35
    ExtendedSourceEncircledEnergy = 36
    SurfaceCurvatureCross = 37
    SurfacePhaseCross = 38
    SurfaceSagCross = 39
    SurfaceCurvature = 40
    SurfacePhase = 41
    SurfaceSag = 42
    StandardSpot = 43
    ThroughFocusSpot = 44
    FullFieldSpot = 45
    MatrixSpot = 46
    ConfigurationMatrixSpot = 47
    RMSField = 48
    RMSFieldMap = 49
    RMSLambdaDiagram = 50
    RMSFocus = 51
    Foucault = 52
    Interferogram = 53
    WavefrontMap = 54
    DetectorViewer = 55
    Draw2D = 56
    Draw3D = 57
    ImageSimulation = 58
    GeometricImageAnalysis = 59
    IMABIMFileViewer = 60
    GeometricBitmapImageAnalysis = 61
    BitmapFileViewer = 62
    LightSourceAnalysis = 63
    PartiallyCoherentImageAnalysis = 64
    ExtendedDiffractionImageAnalysis = 65
    BiocularFieldOfViewAnalysis = 66
    BiocularDipvergenceConvergence = 67
    RelativeIllumination = 68
    VignettingDiagramSettings = 69
    FootprintSettings = 70
    YYbarDiagram = 71
    PowerFieldMapSettings = 72
    PowerPupilMapSettings = 73
    IncidentAnglevsImageHeight = 74
    FiberCouplingSettings = 75
    YNIContributions = 76
    SagTable = 77
    CardinalPoints = 78
    DispersionDiagram = 79
    GlassMap = 80
    AthermalGlassMap = 81
    InternalTransmissionvsWavelength = 82
    DispersionvsWavelength = 83
    GrinProfile = 84
    GradiumProfile = 85
    UniversalPlot1D = 86
    UniversalPlot2D = 87
    PolarizationRayTrace = 88
    PolarizationPupilMap = 89
    Transmission = 90
    PhaseAberration = 91
    TransmissionFan = 92
    ParaxialGaussianBeam = 93
    SkewGaussianBeam = 94
    PhysicalOpticsPropagation = 95
    BeamFileViewer = 96
    ReflectionvsAngle = 97
    TransmissionvsAngle = 98
    AbsorptionvsAngle = 99
    DiattenuationvsAngle = 100
    PhasevsAngle = 101
    RetardancevsAngle = 102
    ReflectionvsWavelength = 103
    TransmissionvsWavelength = 104
    AbsorptionvsWavelength = 105
    DiattenuationvsWavelength = 106
    PhasevsWavelength = 107
    RetardancevsWavelength = 108
    DirectivityPlot = 109
    SourcePolarViewer = 110
    PhotoluminscenceViewer = 111
    SourceSpectrumViewer = 112
    RadiantSourceModelViewerSettings = 113
    SurfaceDataSettings = 114
    PrescriptionDataSettings = 115
    FileComparatorSettings = 116
    PartViewer = 117
    ReverseRadianceAnalysis = 118
    PathAnalysis = 119
    FluxvsWavelength = 120
    RoadwayLighting = 121
    SourceIlluminationMap = 122
    ScatterFunctionViewer = 123
    ScatterPolarPlotSettings = 124
    ZemaxElementDrawing = 125
    ShadedModel = 126
    NSCShadedModel = 127
    NSC3DLayout = 128
    NSCObjectViewer = 129
    RayDatabaseViewer = 130
    ISOElementDrawing = 131
    SystemData = 132
    TestPlateList = 133
    SourceColorChart1931 = 134
    SourceColorChart1976 = 135
    PrescriptionGraphic = 136
    CriticalRayTracer = 137
    ContrastLoss = 138
    CoatingListing = 139
    FullFieldAberration = 140
    SurfaceSlope = 141
    SurfaceSlopeCross = 142
    QuickYield = 143
    SystemCheck = 144
    ToleranceYield = 145
    ToleranceHistogram = 146
    DiffEfficiency2D = 147
    DiffEfficiencyAngular = 148
    DiffEfficiencyChromatic = 149
    NSCSurfaceSag = 150
    NSCSingleRayTrace = 151
    NSCGeometricMtf = 152
    XXXTemplateXXX = -1

class Beam(Enum):
    Reference = 0
    Configuration_1 = 1
    Configuration_2 = 2
    Configuration_3 = 3
    Configuration_4 = 4
    Configuration_5 = 5
    Configuration_6 = 6
    Configuration_7 = 7
    Configuration_8 = 8
    Configuration_9 = 9
    Configuration_10 = 10
    Configuration_11 = 11
    Configuration_12 = 12
    Configuration_13 = 13
    Configuration_14 = 14
    Configuration_15 = 15
    Configuration_16 = 16
    Configuration_17 = 17
    Configuration_18 = 18
    Configuration_19 = 19
    Configuration_20 = 20

class BestFitSphereOptions(Enum):
    MinimumVolume = 0
    MinimumRMS = 1
    MinimumRMSWithOffset = 2

class ColorPaletteType(Enum):
    GreyScale = 0
    FalseColor = 1
    FalseColorOriginal = 2
    Viridis = 3
    Magma = 4

class DetectorViewerShowAsTypes(Enum):
    GreyScale = 0
    FullListing = 0
    Text_CrossSection_Row = 1
    AzimuthCrossSection = 1
    GreyScale_Inverted = 1
    Text_CrossSection_Column = 2
    FalseColor = 2
    FluxVsWaveLength = 3
    FalseColor_Inverted = 3
    TrueColor = 4
    CrossSection_Row = 4
    CrossSection = 5
    Color_CrossSection_Row = 5
    CrossSection_Column = 5
    Directivity_Full = 6
    Color_CrossSection_Column = 6
    GeometricMtf = 6
    Color_FluxVsWavelength = 7
    Directivity_Half = 7

class DetectorViewerShowDataTypes(Enum):
    IncoherentIrradiance = 0
    Polar_AngleSpace = 0
    IncoherentFluence = 0
    IncidentFlux = 0
    PositionSpace = 0
    IncoherentIlluminance = 0
    CoherentFluence = 1
    AngleSpace = 1
    CoherentIrradiance = 1
    AbsorbedFlux = 1
    CoherentIlluminance = 1
    AbsorbedFluxVolume = 2
    CoherentPhase = 2
    RadiantIntensity = 3
    LuminousIntensity = 3
    LuminancePositionSpace = 4
    RadiancePositionSpace = 4
    LuminanceAngleSpace = 5
    RadianceAngleSpace = 5

class ErrorType(Enum):
    Success = 0
    InvalidParameter = 1
    InvalidSettings = 2
    Failed = 3
    AnalysisUnavailableForProgramMode = 4
    NotYetImplemented = 5
    NoSolverLicenseAvailable = 10000
    ToolAlreadyOpen = 10001
    SequentialOnly = 10002
    NonSequentialOnly = 10003
    SingleNSCRayTraceSupported = 10004
    HPCNotAvailable = 10005

class GiaShowAsTypes(Enum):
    Surface = 0
    Contour = 1
    GreyScale = 2
    InverseGreyScale = 3
    FalseColor = 4
    InverseFalseColor = 5
    SpotDiagram = 6
    CrossX = 7
    CrossY = 8

class GridPlotType(Enum):
    Surface = 0
    Contour = 1
    GrayScale = 2
    InverseGrayScale = 3
    FalseColor = 4
    InverseFalseColor = 5

class HuygensShowAsTypes(Enum):
    Surface = 0
    Contour = 1
    GreyScale = 2
    InverseGreyScale = 3
    FalseColor = 4
    InverseFalseColor = 5
    TrueColor = 6

class HuygensSurfaceMftShowAsTypes(Enum):
    GreyScale = 0
    InverseGreyScale = 1
    FalseColor = 2
    InverseFalseColor = 3

class POPSampling(Enum):
    S_32 = 5
    S_64 = 6
    S_128 = 7
    S_256 = 8
    S_512 = 9
    S_1024 = 10
    S_1K = 10
    S_2048 = 11
    S_2K = 11
    S_4K = 12
    S_4096 = 12
    S_8192 = 13
    S_8K = 13
    S_16384 = 14
    S_16K = 14
    S_32K = 15
    S_32768 = 15
    S_65536 = 16
    S_64K = 16
    S_131072 = 17
    S_128K = 17
    S_256K = 18
    S_262144 = 18
    S_524288 = 19
    S_512K = 19
    S_1048576 = 20
    S_1M = 20
    S_2M = 21
    S_2097152 = 21
    S_4194304 = 22
    S_4M = 22
    S_8388608 = 23
    S_8M = 23
    S_16M = 24
    S_16777216 = 24
    S_33554432 = 25
    S_32M = 25
    S_67108864 = 26
    S_64M = 26
    S_128M = 27
    S_134217728 = 27
    S_268435456 = 28
    S_256M = 28
    S_536870912 = 29
    S_512M = 29
    S_1073741824 = 30
    S_1G = 30

class RemoveOptions(Enum):
    None_ = 0
    BaseROC = 1
    BestFitSphere = 2
    BaseSag = 3

class SampleSizes(Enum):
    S_32x32 = 1
    S_64x64 = 2
    S_128x128 = 3
    S_256x256 = 4
    S_512x512 = 5
    S_1024x1024 = 6
    S_2048x2048 = 7
    S_4096x4096 = 8
    S_8192x8192 = 9
    S_16384x16384 = 10

class SampleSizes_ContrastLoss(Enum):
    S_3x3 = 1
    S_5x5 = 2
    S_7x7 = 3
    S_9x9 = 4
    S_11x11 = 5
    S_13x13 = 6
    S_15x15 = 7
    S_17x17 = 8
    S_19x19 = 9
    S_21x21 = 10
    S_23x23 = 11
    S_25x25 = 12
    S_27x27 = 13
    S_29x29 = 14
    S_31x31 = 15
    S_33x33 = 16
    S_35x35 = 17
    S_37x37 = 18
    S_39x39 = 19
    S_41x41 = 20
    S_43x43 = 21
    S_45x45 = 22
    S_47x47 = 23
    S_49x49 = 24
    S_51x51 = 25
    S_53x53 = 26
    S_55x55 = 27
    S_57x57 = 28
    S_59x59 = 29
    S_61x61 = 30
    S_63x63 = 31
    S_65x65 = 32
    S_67x67 = 33
    S_69x69 = 34
    S_71x71 = 35
    S_73x73 = 36
    S_75x75 = 37
    S_77x77 = 38
    S_79x79 = 39
    S_81x81 = 40
    S_83x83 = 41

class SampleSizes_Pow2Plus1(Enum):
    S_33 = 1
    S_65 = 2
    S_129 = 3
    S_257 = 4
    S_513 = 5
    S_1025 = 6
    S_2049 = 7
    S_4097 = 8
    S_8193 = 9

class SampleSizes_Pow2Plus1_X(Enum):
    S_33x33 = 1
    S_65x65 = 2
    S_129x129 = 3
    S_257x257 = 4
    S_513x513 = 5
    S_1025x1025 = 6
    S_2049x2049 = 7
    S_4097x4097 = 8
    S_8193x8193 = 9

class ShowAs(Enum):
    Surface = 0
    Contour = 1
    GreyScale = 2
    InverseGreyScale = 3
    FalseColor = 4
    InverseFalseColor = 5

class SurfaceCurvatureCrossData(Enum):
    TangentialCurvature = 0
    SagitalCurvature = 1
    X_Curvature = 2
    Y_Curvature = 3
    TanPlusSagCurvature = 4
    XPlusYCurvature = 5

class SurfaceCurvatureData(Enum):
    TangentialCurvature = 0
    SagitalCurvature = 1
    X_Curvature = 2
    Y_Curvature = 3

class SurfacePhaseData(Enum):
    SurfacePhase = 0

class SurfaceSagData(Enum):
    SurfaceSag = 0

class SurfaceSlopeCrossData(Enum):
    TangentialCurvature = 0
    SagitalCurvature = 1
    X_Curvature = 2
    Y_Curvature = 3
    TanPlusSagCurvature = 4
    XPlusYCurvature = 5

class SurfaceSlopeData(Enum):
    SurfaceSlope = 0
    TangentialSlope = 0
    SagittalSlope = 1
    XSlope = 2
    YSlope = 3

class UserAnalysisDataType(Enum):
    None_ = 0
    Line2D = 1
    Grid = 2
    GridRGB = 3
    Text = 4



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Common import IMatrixData, IVectorData
from zospy.api._ZOSAPI.Tools import HPCRunState, ISystemTool

__all__ = (
    "CriterionComps",
    "CriterionFields",
    "Criterions",
    "IMonteCarloData",
    "IMonteCarloData",
    "IQuickSensitivity",
    "ISensitivityCompensator",
    "ISensitivityCriterionMetadata",
    "ISensitivityCriterionMetadata",
    "ISensitivityData",
    "ISensitivityData",
    "ISensitivityOperandEffect",
    "ISensitivityOperandEffect",
    "ISensitivityOperandMetadata",
    "ITeziData",
    "IToleranceDataViewer",
    "ITolerancing",
    "ITolerancingColumnMetadata",
    "ITolerancingColumnMetadata",
    "ITolerancingHistogram",
    "ITolerancingHistogram",
    "ITolerancingOperandMetadata",
    "ITolerancingParameter",
    "ITolerancingParameter",
    "ITolerancingSummaryStatistics",
    "ITolerancingSummaryStatistics",
    "MonteCarloStatistics",
    "QSCriterions",
    "SetupCaches",
    "SetupChanges",
    "SetupModes",
    "SetupPolynomials",
    "TolerancingColumnName",
    "TolerancingOperand",
    "TolerancingParameterName",
)

class CriterionComps:
    OptimizeAll_DLS = 0
    NONSEQOptimizeAll = 0
    ParaxialFocus = 1
    NONSEQNone = 1
    NONSEQOptimizeAll_OD = 2
    # None = 2
    OptimizeAll_OD = 3

class CriterionFields:
    NONSEQUserDefined = 0
    Y_Symmetric = 0
    XY_Symmetric = 1
    UserDefined = 2

class Criterions:
    NONSEQMeritFunction = 0
    RMSSpotRadius = 0
    RMSSpotX = 1
    NONSEQUserScript = 1
    RMSSpotY = 2
    RMSWavefront = 3
    MeritFunction = 4
    GeometricMTFAverage = 5
    GeometricMTFTan = 6
    GeometricMTFSag = 7
    DiffMTFAverage = 8
    DiffMTFTan = 9
    DiffMTFSag = 10
    BoresightError = 11
    RMSAngularRadius = 12
    RMSAngularX = 13
    RMSAngularY = 14
    UserScript = 15

class IMonteCarloData:
    @property
    def Values(self) -> IMatrixData: ...
    def GetMetadata(self, columnIndex: int) -> ITolerancingColumnMetadata: ...

class IQuickSensitivity(ISystemTool):
    @property
    def Configuration(self) -> int: ...
    @property
    def Criterion(self) -> QSCriterions: ...
    @property
    def Field(self) -> CriterionFields: ...
    @property
    def MaxSampling(self) -> int: ...
    @property
    def NumberOfConfigurations(self) -> int: ...
    @property
    def ResultsFile(self) -> str: ...
    @property
    def Sampling(self) -> int: ...
    @property
    def ZTDFile(self) -> str: ...
    def LoadSettings(self, settingsFile: str) -> None: ...
    def ResetSettings(self) -> None: ...
    def SaveSettings(self, settingsFile: str) -> None: ...
    @Configuration.setter
    def Configuration(self, value: int) -> None: ...
    @Criterion.setter
    def Criterion(self, value: QSCriterions) -> None: ...
    @Field.setter
    def Field(self, value: CriterionFields) -> None: ...
    @Sampling.setter
    def Sampling(self, value: int) -> None: ...
    @ZTDFile.setter
    def ZTDFile(self, value: str) -> None: ...
    def ValidateCompensators(self) -> bool: ...

class ISensitivityCompensator(ITolerancingOperandMetadata, ITolerancingColumnMetadata):
    @property
    def Maximum(self) -> float: ...
    @property
    def Mean(self) -> float: ...
    @property
    def Minimum(self) -> float: ...
    @property
    def PopulationStandardDeviation(self) -> float: ...
    @property
    def SampleStandardDeviation(self) -> float: ...

class ISensitivityCriterionMetadata:
    @property
    def Name(self) -> Criterions: ...
    @property
    def NominalValue(self) -> float: ...
    @property
    def NumberOfOperands(self) -> int: ...
    def GetEffectByOperand(self, operandIndex: int) -> ISensitivityOperandEffect: ...

class ISensitivityData:
    @property
    def NumberOfCompensators(self) -> int: ...
    @property
    def NumberOfCriteria(self) -> int: ...
    @property
    def NumberOfResultOperands(self) -> int: ...
    def GetCompensator(self, index: int) -> ISensitivityCompensator: ...
    def GetCriterion(self, index: int) -> ISensitivityCriterionMetadata: ...
    def GetOperand(self, index: int) -> ISensitivityOperandMetadata: ...

class ISensitivityOperandEffect:
    @property
    def EstimatedChangeMaximum(self) -> float: ...
    @property
    def EstimatedChangeMinimum(self) -> float: ...

class ISensitivityOperandMetadata(ITolerancingOperandMetadata, ITolerancingColumnMetadata):
    @property
    def Comment(self) -> str: ...
    @property
    def Maximum(self) -> float: ...
    @property
    def Minimum(self) -> float: ...
    @property
    def NumberOfCriteria(self) -> int: ...
    def GetEffectOnCriterion(self, criterionIndex: int) -> ISensitivityOperandEffect: ...

class ITeziData(ITolerancingOperandMetadata, ITolerancingColumnMetadata):
    @property
    def Coefficients(self) -> IMatrixData: ...

class IToleranceDataViewer(ISystemTool):
    @property
    def FileName(self) -> str: ...
    @property
    def MonteCarloData(self) -> IMonteCarloData: ...
    @property
    def SensitivityData(self) -> ISensitivityData: ...
    @property
    def Summary(self) -> str: ...
    def GetToleranceFiles(self) -> list[str]: ...
    @FileName.setter
    def FileName(self, value: str) -> None: ...
    def UseSystemTolerances(self) -> None: ...

class ITolerancing(ISystemTool):
    def EstimateHPCTime(self) -> bool: ...
    @property
    def BestWorstOutputFolder(self) -> str: ...
    @property
    def Criterion(self) -> Criterions: ...
    @property
    def CriterionComp(self) -> CriterionComps: ...
    @property
    def CriterionCompIndex(self) -> int: ...
    @property
    def CriterionConfiguration(self) -> int: ...
    @property
    def CriterionConfigurationIndex(self) -> int: ...
    @property
    def CriterionCycle(self) -> int: ...
    @property
    def CriterionCycleIndex(self) -> int: ...
    @property
    def CriterionField(self) -> CriterionFields: ...
    @property
    def CriterionFieldIndex(self) -> int: ...
    @property
    def CriterionIndex(self) -> int: ...
    @property
    def CriterionSampling(self) -> int: ...
    @property
    def CriterionSamplingIndex(self) -> int: ...
    @property
    def CriterionScript(self) -> int: ...
    @property
    def CriterionScriptIndex(self) -> int: ...
    @property
    def DisplayShowWorst(self) -> int: ...
    @property
    def DisplayShowWorstIndex(self) -> int: ...
    @property
    def FilePrefix(self) -> str: ...
    @property
    def HPCEstimatedTimeS(self) -> float: ...
    @property
    def HPCHasTimeEstimate(self) -> bool: ...
    @property
    def HPCQueuePosition(self) -> int: ...
    @property
    def HPCRemainingTimeS(self) -> float: ...
    @property
    def HPCState(self) -> HPCRunState: ...
    @property
    def HPCTimeToStartS(self) -> float: ...
    @property
    def IsForceRayAimingUsed(self) -> bool: ...
    @property
    def IsHideAllButWorstUsed(self) -> bool: ...
    @property
    def IsHPCEnabled(self) -> bool: ...
    @property
    def IsOverlayGraphicsUsed(self) -> bool: ...
    @property
    def IsSaveBestWorstUsed(self) -> bool: ...
    @property
    def IsSeperateFieldConfigurationsUsed(self) -> bool: ...
    @property
    def IsShowCompensatorsUsed(self) -> bool: ...
    @property
    def IsShowDescriptionsUsed(self) -> bool: ...
    @property
    def MaximumCriteria(self) -> float: ...
    @property
    def MonteCarloStatistic(self) -> MonteCarloStatistics: ...
    @property
    def MonteCarloStatisticIndex(self) -> int: ...
    @property
    def MTFFrequency(self) -> float: ...
    @property
    def NumberOfCriteria(self) -> int: ...
    @property
    def NumberOfCriterionComps(self) -> int: ...
    @property
    def NumberOfCriterionConfigurations(self) -> int: ...
    @property
    def NumberOfCriterionCycles(self) -> int: ...
    @property
    def NumberOfCriterionFields(self) -> int: ...
    @property
    def NumberOfCriterionSamplings(self) -> int: ...
    @property
    def NumberOfCriterionScripts(self) -> int: ...
    @property
    def NumberOfDisplayShowWorsts(self) -> int: ...
    @property
    def NumberOfMonteCarloStatistics(self) -> int: ...
    @property
    def NumberOfRuns(self) -> int: ...
    @property
    def NumberOfSetupCaches(self) -> int: ...
    @property
    def NumberOfSetupChanges(self) -> int: ...
    @property
    def NumberOfSetupCores(self) -> int: ...
    @property
    def NumberOfSetupModes(self) -> int: ...
    @property
    def NumberOfSetupPolynomials(self) -> int: ...
    @property
    def NumberToSave(self) -> int: ...
    @property
    def OpenDataViewer(self) -> bool: ...
    @property
    def OutputFile(self) -> str: ...
    @property
    def ResultFilename(self) -> str: ...
    @property
    def SaveTolDataFile(self) -> bool: ...
    @property
    def SetupCache(self) -> SetupCaches: ...
    @property
    def SetupCacheIndex(self) -> int: ...
    @property
    def SetupChange(self) -> SetupChanges: ...
    @property
    def SetupChangeIndex(self) -> int: ...
    @property
    def SetupCore(self) -> int: ...
    @property
    def SetupCoreIndex(self) -> int: ...
    @property
    def SetupMode(self) -> SetupModes: ...
    @property
    def SetupModeIndex(self) -> int: ...
    @property
    def SetupPolynomial(self) -> SetupPolynomials: ...
    @property
    def SetupPolynomialsIndex(self) -> int: ...
    @property
    def TolDataFile(self) -> str: ...
    @property
    def UseDataRetention(self) -> bool: ...
    def GetCriterionAt(self, idx: int) -> str: ...
    def GetCriterionCompAt(self, idx: int) -> str: ...
    def GetCriterionConfigurationAt(self, idx: int) -> str: ...
    def GetCriterionCycleAt(self, idx: int) -> str: ...
    def GetCriterionFieldAt(self, idx: int) -> str: ...
    def GetCriterionSampleAt(self, idx: int) -> str: ...
    def GetCriterionScriptAt(self, idx: int) -> str: ...
    def GetDisplayShowWorstAt(self, idx: int) -> str: ...
    def GetMonteCarloStatisticAt(self, idx: int) -> str: ...
    def GetSetupCacheAt(self, idx: int) -> str: ...
    def GetSetupChangeAt(self, idx: int) -> str: ...
    def GetSetupCoreAt(self, idx: int) -> str: ...
    def GetSetupModeAt(self, idx: int) -> str: ...
    def GetSetupPolynomialAt(self, idx: int) -> str: ...
    def Load(self, filename: str) -> bool: ...
    def Reset(self) -> bool: ...
    def Save(self, filename: str) -> bool: ...
    @Criterion.setter
    def Criterion(self, value: Criterions) -> None: ...
    @CriterionComp.setter
    def CriterionComp(self, value: CriterionComps) -> None: ...
    @CriterionCompIndex.setter
    def CriterionCompIndex(self, value: int) -> None: ...
    @CriterionConfiguration.setter
    def CriterionConfiguration(self, value: int) -> None: ...
    @CriterionConfigurationIndex.setter
    def CriterionConfigurationIndex(self, value: int) -> None: ...
    @CriterionCycle.setter
    def CriterionCycle(self, value: int) -> None: ...
    @CriterionCycleIndex.setter
    def CriterionCycleIndex(self, value: int) -> None: ...
    @CriterionField.setter
    def CriterionField(self, value: CriterionFields) -> None: ...
    @CriterionFieldIndex.setter
    def CriterionFieldIndex(self, value: int) -> None: ...
    @CriterionIndex.setter
    def CriterionIndex(self, value: int) -> None: ...
    @CriterionSampling.setter
    def CriterionSampling(self, value: int) -> None: ...
    @CriterionSamplingIndex.setter
    def CriterionSamplingIndex(self, value: int) -> None: ...
    @CriterionScript.setter
    def CriterionScript(self, value: int) -> None: ...
    @CriterionScriptIndex.setter
    def CriterionScriptIndex(self, value: int) -> None: ...
    @DisplayShowWorst.setter
    def DisplayShowWorst(self, value: int) -> None: ...
    @DisplayShowWorstIndex.setter
    def DisplayShowWorstIndex(self, value: int) -> None: ...
    @FilePrefix.setter
    def FilePrefix(self, value: str) -> None: ...
    @IsForceRayAimingUsed.setter
    def IsForceRayAimingUsed(self, value: bool) -> None: ...
    @IsHideAllButWorstUsed.setter
    def IsHideAllButWorstUsed(self, value: bool) -> None: ...
    @IsOverlayGraphicsUsed.setter
    def IsOverlayGraphicsUsed(self, value: bool) -> None: ...
    @IsSaveBestWorstUsed.setter
    def IsSaveBestWorstUsed(self, value: bool) -> None: ...
    @IsSeperateFieldConfigurationsUsed.setter
    def IsSeperateFieldConfigurationsUsed(self, value: bool) -> None: ...
    @IsShowCompensatorsUsed.setter
    def IsShowCompensatorsUsed(self, value: bool) -> None: ...
    @IsShowDescriptionsUsed.setter
    def IsShowDescriptionsUsed(self, value: bool) -> None: ...
    @MaximumCriteria.setter
    def MaximumCriteria(self, value: float) -> None: ...
    @MonteCarloStatistic.setter
    def MonteCarloStatistic(self, value: MonteCarloStatistics) -> None: ...
    @MonteCarloStatisticIndex.setter
    def MonteCarloStatisticIndex(self, value: int) -> None: ...
    @MTFFrequency.setter
    def MTFFrequency(self, value: float) -> None: ...
    @NumberOfRuns.setter
    def NumberOfRuns(self, value: int) -> None: ...
    @NumberToSave.setter
    def NumberToSave(self, value: int) -> None: ...
    @OpenDataViewer.setter
    def OpenDataViewer(self, value: bool) -> None: ...
    @OutputFile.setter
    def OutputFile(self, value: str) -> None: ...
    @SaveTolDataFile.setter
    def SaveTolDataFile(self, value: bool) -> None: ...
    @SetupCache.setter
    def SetupCache(self, value: SetupCaches) -> None: ...
    @SetupCacheIndex.setter
    def SetupCacheIndex(self, value: int) -> None: ...
    @SetupChange.setter
    def SetupChange(self, value: SetupChanges) -> None: ...
    @SetupChangeIndex.setter
    def SetupChangeIndex(self, value: int) -> None: ...
    @SetupCore.setter
    def SetupCore(self, value: int) -> None: ...
    @SetupCoreIndex.setter
    def SetupCoreIndex(self, value: int) -> None: ...
    @SetupMode.setter
    def SetupMode(self, value: SetupModes) -> None: ...
    @SetupModeIndex.setter
    def SetupModeIndex(self, value: int) -> None: ...
    @SetupPolynomial.setter
    def SetupPolynomial(self, value: SetupPolynomials) -> None: ...
    @SetupPolynomialsIndex.setter
    def SetupPolynomialsIndex(self, value: int) -> None: ...
    @TolDataFile.setter
    def TolDataFile(self, value: str) -> None: ...
    @UseDataRetention.setter
    def UseDataRetention(self, value: bool) -> None: ...
    def SetPartialMCTMode(self) -> None: ...

class ITolerancingColumnMetadata:
    def AsTeziData(self) -> ITeziData: ...
    @property
    def IsOperand(self) -> bool: ...
    @property
    def Name(self) -> TolerancingColumnName: ...
    @property
    def NumberOfParameters(self) -> int: ...
    @property
    def SummaryStatistics(self) -> ITolerancingSummaryStatistics: ...
    def GetOperandType(self) -> TolerancingOperand: ...
    def GetParameter(self, index: int) -> ITolerancingParameter: ...

class ITolerancingHistogram:
    @property
    def BinCounts(self) -> IVectorData: ...
    @property
    def BinValues(self) -> IVectorData: ...
    @property
    def NumberOfBins(self) -> int: ...
    @property
    def Overflow(self) -> float: ...
    @property
    def Underflow(self) -> float: ...

class ITolerancingOperandMetadata(ITolerancingColumnMetadata):
    @property
    def OperandType(self) -> TolerancingOperand: ...

class ITolerancingParameter:
    @property
    def DoubleValue(self) -> float: ...
    @property
    def IntValue(self) -> int: ...
    @property
    def IsDouble(self) -> bool: ...
    @property
    def IsInt(self) -> bool: ...
    @property
    def IsString(self) -> bool: ...
    @property
    def Name(self) -> TolerancingParameterName: ...
    @property
    def StringValue(self) -> str: ...

class ITolerancingSummaryStatistics:
    @property
    def Histogram(self) -> ITolerancingHistogram: ...
    @property
    def Maximum(self) -> float: ...
    @property
    def Mean(self) -> float: ...
    @property
    def Minimum(self) -> float: ...
    @property
    def PopulationStandardDeviation(self) -> float: ...
    @property
    def SampleError(self) -> float: ...
    @property
    def SampleSize(self) -> int: ...
    @property
    def SampleStandardDeviation(self) -> float: ...
    @property
    def Variance(self) -> float: ...

class MonteCarloStatistics:
    Normal = 0
    Uniform = 1
    Parabolic = 2

class QSCriterions:
    RMSSpotRadius = 0
    RMSSpotX = 1
    RMSSpotY = 2
    RMSWavefront = 3
    BoresightError = 4
    RMSAngularRadius = 5
    RMSAngularX = 6
    RMSAngularY = 7

class SetupCaches:
    RecomputeAll = 0
    RecomputeChanged = 1
    UsePolynomial = 2

class SetupChanges:
    LinearDifference = 0
    RSSDifference = 1

class SetupModes:
    Sensitivity = 0
    InverseLimit = 1
    InverseIncrement = 2
    SkipSensitivity = 3

class SetupPolynomials:
    # None = 0
    ThreeMinusTerm = 1
    FiveMinusTerm = 2

class TolerancingColumnName:
    UserMeritFunction = 11959
    UserScript = 11960
    RmsSpotRadius = 11961
    RmsSpotX = 11962
    RmsSpotY = 11963
    RmsWavefrontError = 11964
    MtfGeometricAverage = 11965
    MtfGeometricTangential = 11966
    MtfGeometricSaggital = 11967
    MtfDiffractionAverage = 11968
    MtfDiffractionTangential = 11969
    MtfDiffractionSagittal = 11970
    BoresightError = 11971
    RmsAngularRadius = 11972
    RmsAngularX = 11973
    RmsAngularY = 11974
    BackFocusChange = 75417
    Unknown = -1

class TolerancingOperand:
    NULL = 0
    TRAD = 1
    TCUR = 2
    TFRN = 3
    TTHI = 4
    TCON = 5
    TSDI = 6
    TSDX = 7
    TSDY = 8
    TSTX = 9
    TSTY = 10
    TIRX = 11
    TIRY = 12
    TIRR = 13
    TPAR = 14
    TEDV = 15
    TEDX = 16
    TEDY = 17
    TETX = 18
    TETY = 19
    TETZ = 20
    TUDX = 21
    TUDY = 22
    TUTX = 23
    TUTY = 24
    TUTZ = 25
    TIND = 26
    TABB = 27
    TEXI = 28
    SAVE = 29
    COMP = 30
    CPAR = 31
    CEDV = 32
    CMCO = 33
    STAT = 34
    TWAV = 35
    TNPS = 36
    TNPA = 37
    TMCO = 38
    TEZI = 39
    TCMU = 40
    SEED = 41
    COMM = 42
    TPAI = 43
    TCIO = 44
    TCEO = 45
    TNMA = 46
    CNPA = 47
    CNPS = 48
    TRLX = 49
    TRLY = 50
    TRLR = 51
    TARX = 52
    TARY = 53
    TARR = 54
    TEDR = 55
    TSDR = 56
    Unknown = -1

class TolerancingParameterName:
    CompensatorSetting = 11400
    ConfigurationNumber = 16304
    CommentField = 20004
    NominalValue = 20566
    Surface = 20567
    MinimumValue = 20568
    MaximumValue = 20569
    Surface1 = 20570
    Surface2 = 20571
    AdjustSurface = 20572
    ParameterNumber = 20573
    Layer = 20574
    MaximumNumber = 20576
    MinimumNumber = 20577
    Code = 20578
    RowNumber = 20579
    ConfigurationParameter = 20580
    Object = 20585
    Data = 20586
    FieldNumber = 20683
    SamplingSetting = 20694
    FieldSetting = 50027
    TdeRowNumber = 75418
    Undefined = -1



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Analysis import IMessage
from zospy.api._ZOSAPI.Common import IVectorData
from zospy.api._ZOSAPI.SystemData import ZemaxSystemUnits
from zospy.api._ZOSAPI.Tools.General import (
    IComputeRMSSpotSize,
    ICreateArchive,
    IExportCAD,
    IHullTools,
    ILensCatalogs,
    IPointTools,
    IQuickAdjust,
    IQuickFocus,
    IRayAimingWizard,
    IRestoreArchive,
    IScale,
)
from zospy.api._ZOSAPI.Tools.Optimization import (
    IGlobalOptimization,
    IHammerOptimization,
    ILocalOptimization,
)
from zospy.api._ZOSAPI.Tools.RayTrace import (
    IBatchRayTrace,
    ILightningTrace,
    INSCRayTrace,
    IZRDReader,
)
from zospy.api._ZOSAPI.Tools.Tolerancing import (
    IQuickSensitivity,
    IToleranceDataViewer,
    ITolerancing,
)

from . import General, LMx, Optimization, RayTrace, Tolerancing

__all__ = (
    "General",
    "LMx",
    "Optimization",
    "RayTrace",
    "Tolerancing",
    "CriticalRayType",
    "HPCEnvironments",
    "HPCNodeSize",
    "HPCRunState",
    "IConvertToNSCGroup",
    "ICriticalRaysetGenerator",
    "IDesignLockdown",
    "IHPCSettings",
    "IHPCSettings",
    "IMaterialsCatalog",
    "IMFCalculator",
    "IOpticalSystemTools",
    "IOpticalSystemTools",
    "IShadedModelTriangleList",
    "IShadedModelTriangleList",
    "IShadedModelVisualizationExport",
    "ISystemTool",
    "ISystemTool",
    "MaterialFormulas",
    "MaterialStatuses",
    "RayPatternOption",
    "RunStatus",
    "VertexOrder",
)

class CriticalRayType:
    Chief = 0
    Marginal = 1
    Grid = 2
    Ring = 3
    Y_Fan = 4
    X_Fan = 5
    XY_Fan = 6
    List = 7

class HPCEnvironments:
    OnPremise = 0
    AWSKubernetes = 1
    AzureKubernetes = 2

class HPCNodeSize:
    Default = 0
    Tiny = 1
    Small = 2
    Medium = 3
    Large = 4
    XLarge = 5

class HPCRunState:
    NotRunning = 0
    Initializing = 1
    ClusterAllocating = 2
    UploadingData = 3
    Queued = 4
    RunStarting = 5
    WaitingForResults = 6
    Complete = 7

class IConvertToNSCGroup(ISystemTool):
    @property
    def ConvertFileToNSC(self) -> bool: ...
    @property
    def ConvertStopToNSCAperture(self) -> bool: ...
    @property
    def ConvertToGlobalCoordinates(self) -> bool: ...
    @property
    def CreateSourcesAndDetectors(self) -> bool: ...
    @property
    def FirstSurface(self) -> int: ...
    @property
    def IgnoreErrors(self) -> bool: ...
    @property
    def LastSurface(self) -> int: ...
    @property
    def OverfillFactor(self) -> float: ...
    @property
    def StopMechanicalHalfWidth(self) -> float: ...
    @property
    def StopObjectFace(self) -> int: ...
    @property
    def StopObjectNumber(self) -> int: ...
    @ConvertFileToNSC.setter
    def ConvertFileToNSC(self, value: bool) -> None: ...
    @ConvertStopToNSCAperture.setter
    def ConvertStopToNSCAperture(self, value: bool) -> None: ...
    @ConvertToGlobalCoordinates.setter
    def ConvertToGlobalCoordinates(self, value: bool) -> None: ...
    @CreateSourcesAndDetectors.setter
    def CreateSourcesAndDetectors(self, value: bool) -> None: ...
    @FirstSurface.setter
    def FirstSurface(self, value: int) -> None: ...
    @IgnoreErrors.setter
    def IgnoreErrors(self, value: bool) -> None: ...
    @LastSurface.setter
    def LastSurface(self, value: int) -> None: ...
    @OverfillFactor.setter
    def OverfillFactor(self, value: float) -> None: ...
    @StopMechanicalHalfWidth.setter
    def StopMechanicalHalfWidth(self, value: float) -> None: ...

class ICriticalRaysetGenerator(ISystemTool):
    @property
    def EffectiveInputDistance(self) -> float: ...
    @property
    def MinimumEfectiveInputDistance(self) -> float: ...
    @property
    def NumRays(self) -> int: ...
    @property
    def RayPattern(self) -> RayPatternOption: ...
    @property
    def RayScale(self) -> float: ...
    @property
    def UseAllFields(self) -> bool: ...
    @property
    def UseAllWavelengths(self) -> bool: ...
    @EffectiveInputDistance.setter
    def EffectiveInputDistance(self, value: float) -> None: ...
    @NumRays.setter
    def NumRays(self, value: int) -> None: ...
    @RayPattern.setter
    def RayPattern(self, value: RayPatternOption) -> None: ...
    @RayScale.setter
    def RayScale(self, value: float) -> None: ...
    @ReadRaysFromFilename.setter
    def ReadRaysFromFilename(self, value: str) -> None: ...
    @SaveCriticalRaysFilename.setter
    def SaveCriticalRaysFilename(self, value: str) -> None: ...
    @UseAllFields.setter
    def UseAllFields(self, value: bool) -> None: ...
    @UseAllWavelengths.setter
    def UseAllWavelengths(self, value: bool) -> None: ...

class IDesignLockdown(ISystemTool):
    @property
    def ConvertSDToMaxApertures(self) -> bool: ...
    @property
    def DecimalPrecision(self) -> int: ...
    @property
    def ExcludePickups(self) -> bool: ...
    @property
    def FixModelGlasses(self) -> bool: ...
    @property
    def UsePrecisionRounding(self) -> bool: ...
    @ConvertSDToMaxApertures.setter
    def ConvertSDToMaxApertures(self, value: bool) -> None: ...
    @DecimalPrecision.setter
    def DecimalPrecision(self, value: int) -> None: ...
    @ExcludePickups.setter
    def ExcludePickups(self, value: bool) -> None: ...
    @FixModelGlasses.setter
    def FixModelGlasses(self, value: bool) -> None: ...
    @UsePrecisionRounding.setter
    def UsePrecisionRounding(self, value: bool) -> None: ...

class IHPCSettings:
    @property
    def InstanceType(self) -> HPCNodeSize: ...
    @property
    def IsHPCConfigured(self) -> bool: ...
    @property
    def MaxNodeCount(self) -> int: ...
    @property
    def NodeCount(self) -> int: ...
    @property
    def ShutdownTimeTicks(self) -> int: ...
    @property
    def ThreadsPerTask(self) -> int: ...

class IMaterialsCatalog(ISystemTool):
    @property
    def AR(self) -> str: ...
    @property
    def CatalogComment(self) -> str: ...
    @property
    def Comment(self) -> str: ...
    @property
    def CR(self) -> str: ...
    @property
    def D0(self) -> float: ...
    @property
    def D1(self) -> float: ...
    @property
    def D2(self) -> float: ...
    @property
    def dPgF(self) -> float: ...
    @property
    def E0(self) -> float: ...
    @property
    def E1(self) -> float: ...
    @property
    def ExcludeSubstitution(self) -> bool: ...
    @property
    def FR(self) -> str: ...
    @property
    def IgnoreThermalExpansion(self) -> bool: ...
    @property
    def Ltk(self) -> float: ...
    @property
    def MaterialFormula(self) -> MaterialFormulas: ...
    @property
    def MaterialStatus(self) -> MaterialStatuses: ...
    @property
    def MaximumWavelength(self) -> float: ...
    @property
    def MeltFreq(self) -> str: ...
    @property
    def MetaMaterial(self) -> bool: ...
    @property
    def MinimumWavelength(self) -> float: ...
    @property
    def Nd(self) -> float: ...
    @property
    def NumberOfFitCoefficients(self) -> int: ...
    @property
    def p(self) -> float: ...
    @property
    def PR(self) -> str: ...
    @property
    def RelCost(self) -> str: ...
    @property
    def SelectedCatalog(self) -> str: ...
    @property
    def SelectedMaterial(self) -> str: ...
    @property
    def SR(self) -> str: ...
    @property
    def TCE(self) -> float: ...
    @property
    def Temp(self) -> float: ...
    @property
    def Vd(self) -> float: ...
    def GetAllCatalogs(self) -> list[str]: ...
    def GetAllMaterials(self) -> list[str]: ...
    def GetCoefficientName(self, coeff: int) -> str: ...
    def GetFitCoefficient(self, coeff: int) -> float: ...
    def SaveCatalog(self) -> bool: ...
    @AR.setter
    def AR(self, value: str) -> None: ...
    @CatalogComment.setter
    def CatalogComment(self, value: str) -> None: ...
    @Comment.setter
    def Comment(self, value: str) -> None: ...
    @CR.setter
    def CR(self, value: str) -> None: ...
    @D0.setter
    def D0(self, value: float) -> None: ...
    @D1.setter
    def D1(self, value: float) -> None: ...
    @D2.setter
    def D2(self, value: float) -> None: ...
    @dPgF.setter
    def dPgF(self, value: float) -> None: ...
    @E0.setter
    def E0(self, value: float) -> None: ...
    @E1.setter
    def E1(self, value: float) -> None: ...
    @ExcludeSubstitution.setter
    def ExcludeSubstitution(self, value: bool) -> None: ...
    @FR.setter
    def FR(self, value: str) -> None: ...
    @IgnoreThermalExpansion.setter
    def IgnoreThermalExpansion(self, value: bool) -> None: ...
    @Ltk.setter
    def Ltk(self, value: float) -> None: ...
    @MaterialFormula.setter
    def MaterialFormula(self, value: MaterialFormulas) -> None: ...
    @MaterialStatus.setter
    def MaterialStatus(self, value: MaterialStatuses) -> None: ...
    @MaximumWavelength.setter
    def MaximumWavelength(self, value: float) -> None: ...
    @MeltFreq.setter
    def MeltFreq(self, value: str) -> None: ...
    @MetaMaterial.setter
    def MetaMaterial(self, value: bool) -> None: ...
    @MinimumWavelength.setter
    def MinimumWavelength(self, value: float) -> None: ...
    @p.setter
    def p(self, value: float) -> None: ...
    @PR.setter
    def PR(self, value: str) -> None: ...
    @RelCost.setter
    def RelCost(self, value: str) -> None: ...
    @SelectedCatalog.setter
    def SelectedCatalog(self, value: str) -> None: ...
    @SelectedMaterial.setter
    def SelectedMaterial(self, value: str) -> None: ...
    @SR.setter
    def SR(self, value: str) -> None: ...
    @TCE.setter
    def TCE(self, value: float) -> None: ...
    @Temp.setter
    def Temp(self, value: float) -> None: ...
    def SetFitCoefficient(self, coeff: int, Value: float) -> None: ...

class IMFCalculator(ISystemTool):
    @property
    def MeritFunctionCalculation(self) -> float: ...

class IOpticalSystemTools:
    def CreateDoubleMatrix(self, Rows: int, Cols: int) -> list[list[float]]: ...
    def CreateDoubleVector(self, numElements: int) -> list[float]: ...
    @property
    def CurrentTool(self) -> ISystemTool: ...
    @property
    def IsRunning(self) -> bool: ...
    @property
    def LastError(self) -> IMessage: ...
    @property
    def Progress(self) -> int: ...
    @property
    def Status(self) -> str: ...
    def GetConversionFromSystemUnits(self, toUnits: ZemaxSystemUnits) -> float: ...
    def GetConversionToSystemUnits(self, fromUnits: ZemaxSystemUnits) -> float: ...
    def GetHPCSettingsCustom(self, nodeCount: int, instanceType: HPCNodeSize) -> IHPCSettings: ...
    def GetHPCSettingsFromPrefs(self) -> IHPCSettings: ...
    def GetHullTools(self) -> IHullTools: ...
    def GetPointTools(self) -> IPointTools: ...
    def OpenBatchRayTrace(self) -> IBatchRayTrace: ...
    def OpenConvertToNSCGroup(self) -> IConvertToNSCGroup: ...
    def OpenCreateZAR(self) -> ICreateArchive: ...
    def OpenCriticalRaysetGenerator(self) -> ICriticalRaysetGenerator: ...
    def OpenDesignLockdown(self) -> IDesignLockdown: ...
    def OpenExportCAD(self) -> IExportCAD: ...
    def OpenGlobalOptimization(self) -> IGlobalOptimization: ...
    def OpenGlobalOptimization_HPC(self, hpcSettings: IHPCSettings) -> IGlobalOptimization: ...
    def OpenHammerOptimization(self) -> IHammerOptimization: ...
    def OpenHammerOptimization_HPC(self, hpcSettings: IHPCSettings) -> IHammerOptimization: ...
    def OpenLensCatalogs(self) -> ILensCatalogs: ...
    def OpenLightningTrace(self) -> ILightningTrace: ...
    def OpenLocalOptimization(self) -> ILocalOptimization: ...
    def OpenMaterialsCatalog(self) -> IMaterialsCatalog: ...
    def OpenMeritFunctionCalculator(self) -> IMFCalculator: ...
    def OpenNSCRayTrace(self) -> INSCRayTrace: ...
    def OpenNSCRayTrace_HPC(self, hpcSettings: IHPCSettings) -> INSCRayTrace: ...
    def OpenQuickAdjust(self) -> IQuickAdjust: ...
    def OpenQuickFocus(self) -> IQuickFocus: ...
    def OpenQuickSensitivity(self) -> IQuickSensitivity: ...
    def OpenRayAimingWizard(self) -> IRayAimingWizard: ...
    def OpenRayDatabaseReader(self) -> IZRDReader: ...
    def OpenRestoreZAR(self) -> IRestoreArchive: ...
    def OpenRMSSpotRadiusCalculator(self) -> IComputeRMSSpotSize: ...
    def OpenScale(self) -> IScale: ...
    def OpenShadedModelVisualizationExport(self) -> IShadedModelVisualizationExport: ...
    def OpenToleranceDataViewer(self) -> IToleranceDataViewer: ...
    def OpenTolerancing(self) -> ITolerancing: ...
    def OpenTolerancing_HPC(self, hpcSettings: IHPCSettings) -> ITolerancing: ...
    def RemoveAllVariables(self) -> bool: ...
    def SetAllRadiiVariable(self) -> int: ...
    def SetAllThicknessesVariable(self) -> int: ...

class IShadedModelTriangleList:
    @property
    def NumberOfSurfaces(self) -> int: ...
    @property
    def NumberOfTriangles(self) -> int: ...
    @property
    def ObjectIndex(self) -> IVectorData: ...
    @property
    def SurfaceIndex(self) -> IVectorData: ...
    @property
    def TotalNumberOfObjects(self) -> int: ...
    @property
    def XCoords(self) -> IVectorData: ...
    @property
    def XNormals(self) -> IVectorData: ...
    @property
    def YCoords(self) -> IVectorData: ...
    @property
    def YNormals(self) -> IVectorData: ...
    @property
    def ZCoords(self) -> IVectorData: ...
    @property
    def ZNormals(self) -> IVectorData: ...
    def GetVertex(
        self, TriangleNumber: int, Vertex: VertexOrder
    ) -> tuple[bool, float, float, float, float, float, float, int, int]: ...
    def NumberOfObjects(self, Surface: int) -> list[int]: ...
    @ObjectIndex.setter
    def ObjectIndex(self, value: IVectorData) -> None: ...
    @SurfaceIndex.setter
    def SurfaceIndex(self, value: IVectorData) -> None: ...
    @XCoords.setter
    def XCoords(self, value: IVectorData) -> None: ...
    @XNormals.setter
    def XNormals(self, value: IVectorData) -> None: ...
    @YCoords.setter
    def YCoords(self, value: IVectorData) -> None: ...
    @YNormals.setter
    def YNormals(self, value: IVectorData) -> None: ...
    @ZCoords.setter
    def ZCoords(self, value: IVectorData) -> None: ...
    @ZNormals.setter
    def ZNormals(self, value: IVectorData) -> None: ...

class IShadedModelVisualizationExport(ISystemTool):
    @property
    def EndSurface(self) -> int: ...
    @property
    def Object(self) -> int: ...
    @property
    def StartSurface(self) -> int: ...
    def GetResults(self) -> IShadedModelTriangleList: ...
    @EndSurface.setter
    def EndSurface(self, value: int) -> None: ...
    @Object.setter
    def Object(self, value: int) -> None: ...
    @StartSurface.setter
    def StartSurface(self, value: int) -> None: ...

class ISystemTool:
    def Cancel(self) -> bool: ...
    def Close(self) -> bool: ...
    @property
    def CanCancel(self) -> bool: ...
    @property
    def ErrorMessage(self) -> str: ...
    @property
    def IsAsynchronous(self) -> bool: ...
    @property
    def IsFiniteDuration(self) -> bool: ...
    @property
    def IsRunning(self) -> bool: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def Progress(self) -> int: ...
    @property
    def Status(self) -> str: ...
    @property
    def Succeeded(self) -> bool: ...
    def Run(self) -> bool: ...
    def RunAndWaitForCompletion(self) -> bool: ...
    def RunAndWaitWithTimeout(self, timeOutSeconds: float) -> RunStatus: ...
    def WaitForCompletion(self) -> bool: ...
    def WaitWithTimeout(self, timeOutSeconds: float) -> RunStatus: ...

class MaterialFormulas:
    Schott = 1
    Sellmeier1 = 2
    Herzberger = 3
    Sellmeier2 = 4
    Conrady = 5
    Sellmeier3 = 6
    Handbook1 = 7
    Handbook2 = 8
    Sellmeier4 = 9
    Extended = 10
    Sellmeier5 = 11
    Extended2 = 12
    Extended3 = 13

class MaterialStatuses:
    Standard = 0
    Preferred = 1
    Obsolete = 2
    Special = 3
    Melt = 4

class RayPatternOption:
    XyFan = 0
    XFan = 1
    YFan = 2
    ChiefAndRing = 3
    List = 4
    Grid = 6
    ChiefAndMarginals = 8

class RunStatus:
    Completed = 0
    FailedToStart = 1
    TimedOut = 2
    InvalidTimeout = 3

class VertexOrder:
    First = 0
    Second = 1
    Third = 2



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = (
    "ApertureShapes",
    "ArrayMode",
    "BirefringentMode",
    "BirefringentReflections",
    "DetectorDataType",
    "DetectorShowAsType",
    "DiffractionSplitType",
    "DiffractiveFaceChoices",
    "DrawingResolutionType",
    "EfficiencySpectrumType",
    "EndCapOptions",
    "FaceIsType",
    "HologramTypes",
    "InterpolateChoices",
    "NCEIndexType",
    "ObjectColumn",
    "ObjectScatteringTypes",
    "ObjectType",
    "OrderChoices",
    "PixelAddressing",
    "PolarDetectorDataType",
    "RaysIgnoreObjectType",
    "RayTraceModes",
    "ScatterToType",
    "ShapeChoices",
    "SourceBulkScatterMode",
    "SourceColorMode",
    "SourceSamplingMethod",
    "UniformAngleChoices",
    "VolumePhysicsModelType",
)

class ApertureShapes(Enum):
    Annular = 0
    Elliptical = 1
    Rectangular = 2
    CyliderWithRectangular = 3

class ArrayMode(Enum):
    None_ = 0
    Rectangular = 1
    Circular = 2
    Hexapolar = 3
    Hexagonal = 4

class BirefringentMode(Enum):
    TraceOrdinayAndExtraordinary = 0
    TraceOrdinaryOnly = 1
    TraceExtraordinaryOnly = 2
    Waveplate = 3

class BirefringentReflections(Enum):
    TraceReflectedAndRefracted = 0
    TraceRefractedOnly = 1
    TraceReflectedOnly = 2

class DetectorDataType(Enum):
    Real = 0
    Imaginary = 1
    Amplitude = 2
    Power = 3

class DetectorShowAsType(Enum):
    GreyScaleFlux = 0
    InverseGreyScaleFlux = 1
    FalseColorFlux = 2
    InverseFalseColorFlux = 3
    GreyScaleIrradiance = 4
    InverseGreyScaleIrradiance = 5
    FalseColorIrradiance = 6
    InverseFalseColorIrradiance = 7

class DiffractionSplitType(Enum):
    DontSplitByOrder = 0
    SplitByTable = 1
    SplitByDLL = 2

class DiffractiveFaceChoices(Enum):
    FrontFace = 1
    BackFace = 2

class DrawingResolutionType(Enum):
    Standard = 0
    Medium = 1
    High = 2
    Presentation = 3
    Custom = 4

class EfficiencySpectrumType(Enum):
    QuantumYield = 0
    Excitation = 1

class EndCapOptions(Enum):
    None_ = 0
    First = 1
    Second = 2
    Both = 3

class FaceIsType(Enum):
    ObjectDefault = 0
    Reflective = 1
    Absorbing = 2

class HologramTypes(Enum):
    Type_1 = 1
    Type_2 = 2

class InterpolateChoices(Enum):
    Bicubic = 0
    Linear = 1

class NCEIndexType(Enum):
    Isotropic = 0
    Birefringent = 1
    GRIN = 2

class ObjectColumn(Enum):
    Comment = 1
    RefObject = 2
    InsideOf = 3
    XPosition = 4
    YPosition = 5
    ZPosition = 6
    TiltX = 7
    TiltY = 8
    TiltZ = 9
    Material = 10
    Par1 = 11
    Par2 = 12
    Par3 = 13
    Par4 = 14
    Par5 = 15
    Par6 = 16
    Par7 = 17
    Par8 = 18
    Par9 = 19
    Par10 = 20
    Par11 = 21
    Par12 = 22
    Par13 = 23
    Par14 = 24
    Par15 = 25
    Par16 = 26
    Par17 = 27
    Par18 = 28
    Par19 = 29
    Par20 = 30
    Par21 = 31
    Par22 = 32
    Par23 = 33
    Par24 = 34
    Par25 = 35
    Par26 = 36
    Par27 = 37
    Par28 = 38
    Par29 = 39
    Par30 = 40
    Par31 = 41
    Par32 = 42
    Par33 = 43
    Par34 = 44
    Par35 = 45
    Par36 = 46
    Par37 = 47
    Par38 = 48
    Par39 = 49
    Par40 = 50
    Par41 = 51
    Par42 = 52
    Par43 = 53
    Par44 = 54
    Par45 = 55
    Par46 = 56
    Par47 = 57
    Par48 = 58
    Par49 = 59
    Par50 = 60
    Par51 = 61
    Par52 = 62
    Par53 = 63
    Par54 = 64
    Par55 = 65
    Par56 = 66
    Par57 = 67
    Par58 = 68
    Par59 = 69
    Par60 = 70
    Par61 = 71
    Par62 = 72
    Par63 = 73
    Par64 = 74
    Par65 = 75
    Par66 = 76
    Par67 = 77
    Par68 = 78
    Par69 = 79
    Par70 = 80
    Par71 = 81
    Par72 = 82
    Par73 = 83
    Par74 = 84
    Par75 = 85
    Par76 = 86
    Par77 = 87
    Par78 = 88
    Par79 = 89
    Par80 = 90
    Par81 = 91
    Par82 = 92
    Par83 = 93
    Par84 = 94
    Par85 = 95
    Par86 = 96
    Par87 = 97
    Par88 = 98
    Par89 = 99
    Par90 = 100
    Par91 = 101
    Par92 = 102
    Par93 = 103
    Par94 = 104
    Par95 = 105
    Par96 = 106
    Par97 = 107
    Par98 = 108
    Par99 = 109
    Par100 = 110
    Par101 = 111
    Par102 = 112
    Par103 = 113
    Par104 = 114
    Par105 = 115
    Par106 = 116
    Par107 = 117
    Par108 = 118
    Par109 = 119
    Par110 = 120
    Par111 = 121
    Par112 = 122
    Par113 = 123
    Par114 = 124
    Par115 = 125
    Par116 = 126
    Par117 = 127
    Par118 = 128
    Par119 = 129
    Par120 = 130
    Par121 = 131
    Par122 = 132
    Par123 = 133
    Par124 = 134
    Par125 = 135
    Par126 = 136
    Par127 = 137
    Par128 = 138
    Par129 = 139
    Par130 = 140
    Par131 = 141
    Par132 = 142
    Par133 = 143
    Par134 = 144
    Par135 = 145
    Par136 = 146
    Par137 = 147
    Par138 = 148
    Par139 = 149
    Par140 = 150
    Par141 = 151
    Par142 = 152
    Par143 = 153
    Par144 = 154
    Par145 = 155
    Par146 = 156
    Par147 = 157
    Par148 = 158
    Par149 = 159
    Par150 = 160
    Par151 = 161
    Par152 = 162
    Par153 = 163
    Par154 = 164
    Par155 = 165
    Par156 = 166
    Par157 = 167
    Par158 = 168
    Par159 = 169
    Par160 = 170
    Par161 = 171
    Par162 = 172
    Par163 = 173
    Par164 = 174
    Par165 = 175
    Par166 = 176
    Par167 = 177
    Par168 = 178
    Par169 = 179
    Par170 = 180
    Par171 = 181
    Par172 = 182
    Par173 = 183
    Par174 = 184
    Par175 = 185
    Par176 = 186
    Par177 = 187
    Par178 = 188
    Par179 = 189
    Par180 = 190
    Par181 = 191
    Par182 = 192
    Par183 = 193
    Par184 = 194
    Par185 = 195
    Par186 = 196
    Par187 = 197
    Par188 = 198
    Par189 = 199
    Par190 = 200
    Par191 = 201
    Par192 = 202
    Par193 = 203
    Par194 = 204
    Par195 = 205
    Par196 = 206
    Par197 = 207
    Par198 = 208
    Par199 = 209
    Par200 = 210
    Par201 = 211
    Par202 = 212
    Par203 = 213
    Par204 = 214
    Par205 = 215
    Par206 = 216
    Par207 = 217
    Par208 = 218
    Par209 = 219
    Par210 = 220
    Par211 = 221
    Par212 = 222
    Par213 = 223
    Par214 = 224
    Par215 = 225
    Par216 = 226
    Par217 = 227
    Par218 = 228
    Par219 = 229
    Par220 = 230
    Par221 = 231
    Par222 = 232
    Par223 = 233
    Par224 = 234
    Par225 = 235
    Par226 = 236
    Par227 = 237
    Par228 = 238
    Par229 = 239
    Par230 = 240
    Par231 = 241
    Par232 = 242
    Par233 = 243
    Par234 = 244
    Par235 = 245
    Par236 = 246
    Par237 = 247
    Par238 = 248
    Par239 = 249
    Par240 = 250
    Par241 = 251
    Par242 = 252
    Par243 = 253
    Par244 = 254
    Par245 = 255
    Par246 = 256
    Par247 = 257
    Par248 = 258
    Par249 = 259
    Par250 = 260

class ObjectScatteringTypes(Enum):
    None_ = 0
    Lambertian = 1
    Gaussian = 2
    ABg = 3
    ABgFile = 4
    BSDF = 5
    ISScatterCatalog = 6
    User = 7

class ObjectType(Enum):
    AnnularAsphericLens = 0
    AnnularAxialLens = 1
    AnnularVolume = 2
    Annulus = 3
    Array = 4
    ArrayRing = 5
    AsphericSurface = 6
    AsphericSurface2 = 7
    AxiconSurface = 8
    BiconicLens = 9
    BiconicSurface = 10
    BiconicZernike = 11
    BiconicZernikeSurface = 12
    Binary1 = 13
    Binary2 = 14
    Binary2A = 15
    Boolean = 16
    BooleanCAD = 16
    CADAssemblyAutodeskInventor = 17
    CADAssemblyCreoParametric = 18
    CADAssemblySolidWorks = 19
    CADPartAutodeskInventor = 20
    CADPartCreoParametric = 21
    CADPartSolidWorks = 22
    CADPartSTEPIGESSAT = 23
    CADPartSTL = 24
    CADPartZPD = 25
    Cone = 26
    CPC = 27
    CPCRectangular = 28
    CylinderPipe = 29
    CylinderVolume = 30
    Cylinder2Pipe = 31
    Cylinder2Volume = 32
    DetectorColor = 33
    DetectorPolar = 34
    DetectorRectangle = 35
    DetectorSurface = 36
    DetectorVolume = 37
    DiffractionGrating = 38
    DualBEFSurface = 39
    Ellipse = 40
    EllipticalVolume = 41
    EvenAsphereLens = 42
    ExtendedOddAsphereLens = 43
    ExtendedPolynomialLens = 44
    ExtendedPolynomialSurface = 45
    Extruded = 46
    FacetedSurface = 47
    FreeformZ = 48
    Fresnel1 = 49
    Fresnel2 = 50
    GridSagLens = 51
    GridSagSurface = 52
    GridSagFrontBack = 53
    HexagonalLensletArray = 54
    HologramLens = 55
    HologramSurface = 56
    JonesMatrix = 57
    LensletArray1 = 58
    LensletArray2 = 59
    MEMS = 60
    NullObject = 61
    OddAsphereLens = 62
    ParaxialLens = 63
    PolygonObject = 64
    RayRotator = 65
    RectangularCorner = 66
    Rectangle = 67
    RectangularPipe = 68
    RectangularPipeGrating = 69
    RectangularRoof = 70
    RectangularTorusSurface = 71
    RectangularTorusVolume = 72
    RectangularVolume = 73
    RectangularVolumeGrating = 74
    ReverseRadianceDetector = 75
    ReverseRadianceTarget = 76
    Slide = 77
    SourceDiffractive = 78
    SourceDiode = 79
    SourceDLL = 80
    SourceEllipse = 81
    SourceEULUMDATFile = 82
    SourceFilament = 83
    SourceFile = 84
    SourceGaussian = 85
    SourceIESNAFile = 86
    SourceImported = 87
    SourceObject = 88
    SourcePoint = 89
    SourceRadial = 90
    SourceRay = 91
    SourceRectangle = 92
    SourceTube = 93
    SourceTwoAngle = 94
    SourceVolumeCylindrical = 95
    SourceVolumeElliptical = 96
    SourceVolumeRectangular = 97
    Sphere = 98
    StandardLens = 99
    StandardSurface = 100
    Swept = 101
    TabulatedFactedRadial = 102
    TabulatedFactedToroid = 103
    TabulatedFrenselRadial = 104
    ToroidalHologram = 105
    ToroidalLens = 106
    ToroidalSurface = 107
    ToroidalSurfaceOddASphere = 108
    TorusSurface = 109
    TorusVolume = 110
    TriangularCorner = 111
    Triangle = 112
    UserDefinedObject = 113
    WolterSurface = 114
    ZernikeSurface = 115
    BooleanNative = 116
    CompoundLens = 117
    QTypeAsphereSurface = 118
    OffAxisMirror = 119

class OrderChoices(Enum):
    Before = 0
    After = 1

class PixelAddressing(Enum):
    ByRow = 0
    ByColumn = 1
    Individually = 2

class PolarDetectorDataType(Enum):
    Power = 1
    PowerSolidAngle = 2
    Lumens = 3
    LumensSolidAngle = 4
    Cx = 5
    Cy = 6
    u_T = 7
    u_V = 8
    TriX = 9
    TriY = 10
    TriZ = 11

class RaysIgnoreObjectType(Enum):
    Never = 0
    Always = 1
    OnLaunch = 2

class RayTraceModes(Enum):
    Standard = 0
    Flat = 1
    Shaded = 2
    Kernel = 3

class ScatterToType(Enum):
    ScatterToList = 0
    ImportanceSampling = 1

class ShapeChoices(Enum):
    Rectangular = 0
    Elliptical = 1

class SourceBulkScatterMode(Enum):
    Many = 0
    Once = 1
    Never = 2

class SourceColorMode(Enum):
    SystemWavelengths = 0
    CIE1931Tristimulus = 1
    CIE1931Chromaticity = 2
    CIE1931RGB = 3
    UniformPowerSpectrum = 4
    D65White = 5
    ColorTemperature = 6
    BlackBodySpectrum = 7
    SpectrumFile = 8
    CIE1976 = 9

class SourceSamplingMethod(Enum):
    Random = 0
    Sobol = 1

class UniformAngleChoices(Enum):
    UniformIrradiance = 0
    UniformInAngleSpace = 1

class VolumePhysicsModelType(Enum):
    None_ = 0
    AngleScattering = 1
    DLLDefinedScattering = 2
    PhotoluminescenceModel = 3



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = (
    "ACISExportVersion",
    "ArchiveFileStatus",
    "CADAngularToleranceType",
    "CADFileType",
    "CADToleranceType",
    "DataTypes",
    "EntryCompressionModes",
    "LensShape",
    "LensType",
    "QuickAdjustCriterion",
    "QuickAdjustType",
    "QuickFocusCriterion",
    "RayPatternType",
    "ScaleToUnits",
    "SplineSegmentsType",
    "ZemaxFileTypes",
)

class ACISExportVersion(Enum):
    Current = 0
    V25 = 1
    V26 = 2
    V27 = 3
    V28 = 4
    V29 = 5
    V30 = 6

class ArchiveFileStatus(Enum):
    Okay = 0
    InvalidVersion = -3
    InvalidFile = -2
    UnableToOpen = -1

class CADAngularToleranceType(Enum):
    Low = 0
    Medium = 1
    High = 2
    Presentation = 3

class CADFileType(Enum):
    IGES = 0
    STEP = 1
    SAT = 2
    STL = 3
    SAB = 4
    ASAT = 5
    ASAB = 6
    MODEL = 7
    CATPART = 8
    CATPRODUCT = 9
    XCGM = 10

class CADToleranceType(Enum):
    N_TenEMinus4 = 0
    N_TenEMinus5 = 1
    N_TenEMinus6 = 2
    N_TenEMinus7 = 3

class DataTypes(Enum):
    NotSet = 0
    Boolean = 1
    Integer = 2
    IntegerArray = 3
    IntegerMatrix = 4
    Float = 5
    FloatArray = 6
    FloatMatrix = 7
    Double = 8
    DoubleArray = 9
    DoubleMatrix = 10
    String = 11
    StringArray = 12
    StringMatrix = 13
    ByteArray = 14
    Dictionary = 15
    Serializable = 16
    File = 17

class EntryCompressionModes(Enum):
    Auto = 0
    On = 1
    Off = 2

class LensShape(Enum):
    Unknown = 0
    Equi = 1
    Bi = 2
    Plano = 3
    Meniscus = 4

class LensType(Enum):
    Other = 0
    Spherical = 1
    GRIN = 2
    Aspheric = 3
    Toroidal = 4

class QuickAdjustCriterion(Enum):
    SpotSizeRadial = 0
    SpotSizeXOnly = 1
    SpotSizeYOnly = 2
    AngularRadial = 3
    AngularXOnly = 4
    AngularYOnly = 5

class QuickAdjustType(Enum):
    Radius = 0
    Thickness = 1

class QuickFocusCriterion(Enum):
    SpotSizeRadial = 0
    SpotSizeXOnly = 1
    SpotSizeYOnly = 2
    RMSWavefront = 3

class RayPatternType(Enum):
    XYFan = 0
    XFan = 1
    YFan = 2
    Ring = 3
    List = 4
    Random = 5
    Grid = 6
    SolidRing = 7

class ScaleToUnits(Enum):
    Millimeters = 0
    Centimeters = 1
    Inches = 2
    Meters = 3

class SplineSegmentsType(Enum):
    N_016 = 0
    N_032 = 1
    N_064 = 2
    N_128 = 3
    N_256 = 4
    N_512 = 5

class ZemaxFileTypes(Enum):
    Unknown = 0
    LMX = 1
    User = 2



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = ("OptimizationAlgorithm", "OptimizationCycles", "OptimizationSaveCount")

class OptimizationAlgorithm(Enum):
    DampedLeastSquares = 0
    OrthogonalDescent = 1

class OptimizationCycles(Enum):
    Automatic = 0
    Fixed_1_Cycle = 1
    Fixed_5_Cycles = 2
    Fixed_10_Cycles = 3
    Fixed_50_Cycles = 4
    Infinite = 5

class OptimizationSaveCount(Enum):
    Save_10 = 0
    Save_20 = 1
    Save_30 = 2
    Save_40 = 3
    Save_50 = 4
    Save_60 = 5
    Save_70 = 6
    Save_80 = 7
    Save_90 = 8
    Save_100 = 9



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Analysis import HuygensShowAsTypes, SampleSizes
from zospy.api._ZOSAPI.Analysis.Settings import (
    IAS_,
    HuygensPsfTypes,
    IAS_Field,
    IAS_Surface,
    IAS_Wavelength,
    PsfSpread,
    PsfTypes,
    Rotations,
)

__all__ = (
    "FftPsfType",
    "IAS_FftPsf",
    "IAS_FftPsfCrossSection",
    "IAS_FftPsfLineEdgeSpread",
    "IAS_HuygensPsf",
    "IAS_HuygensPsfCrossSection",
    "PsfRotation",
    "PsfSampling",
)

class FftPsfType:
    Linear = 0
    Log = 1
    Phase = 2
    Real = 3
    Imaginary = 4

class IAS_FftPsf(IAS_):
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def ImageDelta(self) -> float: ...
    @property
    def Normalize(self) -> bool: ...
    @property
    def OutputSize(self) -> PsfSampling: ...
    @property
    def Rotation(self) -> PsfRotation: ...
    @property
    def SampleSize(self) -> PsfSampling: ...
    @property
    def Surface(self) -> IAS_Surface: ...
    @property
    def Type(self) -> FftPsfType: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @ImageDelta.setter
    def ImageDelta(self, value: float) -> None: ...
    @Normalize.setter
    def Normalize(self, value: bool) -> None: ...
    @OutputSize.setter
    def OutputSize(self, value: PsfSampling) -> None: ...
    @Rotation.setter
    def Rotation(self, value: PsfRotation) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: PsfSampling) -> None: ...
    @Type.setter
    def Type(self, value: FftPsfType) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_FftPsfCrossSection(IAS_):
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def Normalize(self) -> bool: ...
    @property
    def PlotScale(self) -> float: ...
    @property
    def RowCol(self) -> int: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def Type(self) -> PsfTypes: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @Normalize.setter
    def Normalize(self, value: bool) -> None: ...
    @PlotScale.setter
    def PlotScale(self, value: float) -> None: ...
    @RowCol.setter
    def RowCol(self, value: int) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @Type.setter
    def Type(self, value: PsfTypes) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_FftPsfLineEdgeSpread(IAS_):
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def PlotScale(self) -> float: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def Spread(self) -> PsfSpread: ...
    @property
    def Type(self) -> PsfTypes: ...
    @property
    def UseCoherentPSF(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @PlotScale.setter
    def PlotScale(self, value: float) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @Spread.setter
    def Spread(self, value: PsfSpread) -> None: ...
    @Type.setter
    def Type(self, value: PsfTypes) -> None: ...
    @UseCoherentPSF.setter
    def UseCoherentPSF(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_HuygensPsf(IAS_):
    @property
    def Configuration(self) -> int: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def ImageDelta(self) -> float: ...
    @property
    def ImageSampleSize(self) -> SampleSizes: ...
    @property
    def Normalize(self) -> bool: ...
    @property
    def PupilSampleSize(self) -> SampleSizes: ...
    @property
    def Rotation(self) -> Rotations: ...
    @property
    def ShowAsType(self) -> HuygensShowAsTypes: ...
    @property
    def Type(self) -> HuygensPsfTypes: ...
    @property
    def UseCentroid(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @Configuration.setter
    def Configuration(self, value: int) -> None: ...
    @ImageDelta.setter
    def ImageDelta(self, value: float) -> None: ...
    @ImageSampleSize.setter
    def ImageSampleSize(self, value: SampleSizes) -> None: ...
    @Normalize.setter
    def Normalize(self, value: bool) -> None: ...
    @PupilSampleSize.setter
    def PupilSampleSize(self, value: SampleSizes) -> None: ...
    @Rotation.setter
    def Rotation(self, value: Rotations) -> None: ...
    @ShowAsType.setter
    def ShowAsType(self, value: HuygensShowAsTypes) -> None: ...
    @Type.setter
    def Type(self, value: HuygensPsfTypes) -> None: ...
    @UseCentroid.setter
    def UseCentroid(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_HuygensPsfCrossSection(IAS_):
    @property
    def Configuration(self) -> int: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def ImageDelta(self) -> float: ...
    @property
    def ImageSampleSize(self) -> SampleSizes: ...
    @property
    def Normalize(self) -> bool: ...
    @property
    def PupilSampleSize(self) -> SampleSizes: ...
    @property
    def RowCol(self) -> int: ...
    @property
    def Type(self) -> PsfTypes: ...
    @property
    def UseCentroid(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @Configuration.setter
    def Configuration(self, value: int) -> None: ...
    @ImageDelta.setter
    def ImageDelta(self, value: float) -> None: ...
    @ImageSampleSize.setter
    def ImageSampleSize(self, value: SampleSizes) -> None: ...
    @Normalize.setter
    def Normalize(self, value: bool) -> None: ...
    @PupilSampleSize.setter
    def PupilSampleSize(self, value: SampleSizes) -> None: ...
    @RowCol.setter
    def RowCol(self, value: int) -> None: ...
    @Type.setter
    def Type(self, value: PsfTypes) -> None: ...
    @UseCentroid.setter
    def UseCentroid(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class PsfRotation:
    CW0 = 0
    CW90 = 1
    CW180 = 2
    CW270 = 3

class PsfSampling:
    PsfS_32x32 = 1
    PsfS_64x64 = 2
    PsfS_128x128 = 3
    PsfS_256x256 = 4
    PsfS_512x512 = 5
    PsfS_1024x1024 = 6
    PsfS_2048x2048 = 7
    PsfS_4096x4096 = 8
    PsfS_8192x8192 = 9
    PsfS_16384x16384 = 10



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = (
    "POPBeamTypes",
    "POPDataTypes",
    "POPFiberPositions",
    "POPFiberTypes",
    "POPProjectionTypes",
    "POPScaleTypes",
    "POPShowAsTypes",
    "POPZoomTypes",
)

class POPBeamTypes(Enum):
    GaussianWaist = 0
    GaussianAngle = 1
    GaussianSizeAngle = 2
    TopHat = 3
    File = 4
    DLL = 5
    Multimode = 6
    AstigmaticGaussian = 7

class POPDataTypes(Enum):
    Irradiance = 0
    EXIrradiance = 1
    EYIrradiance = 2
    Phase = 3
    EXPhase = 4
    EYPhase = 5
    TransferMagnitude = 6
    TransferPhase = 7

class POPFiberPositions(Enum):
    ChiefRay = 0
    SurfaceVertex = 1

class POPFiberTypes(Enum):
    GaussianWaist = 0
    GaussianAngle = 1
    GaussianSizeAngle = 2
    TopHat = 3
    File = 4
    DLL = 5
    AstigmaticGaussian = 6

class POPProjectionTypes(Enum):
    AlongBeam = 0
    AlongNormal = 1
    AlongLocalZ = 2

class POPScaleTypes(Enum):
    Linear = 0
    Log_Minus_5 = 1
    Log_Minus_10 = 2
    Log_Minus_15 = 3

class POPShowAsTypes(Enum):
    Surface = 0
    Contour = 1
    GrayScale = 2
    InverseGrayScale = 3
    FalseColor = 4
    InverseFalseColor = 5
    CrossX = 6
    CrossY = 7
    Encircled = 8
    Ensquared = 9
    EnslittedX = 10
    EnslittedY = 11

class POPZoomTypes(Enum):
    NoZoom = 0
    X2 = 1
    X4 = 2
    X8 = 3
    X16 = 4



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

from . import RMSField, RMSFieldMap

__all__ = (
    "RMSField",
    "RMSFieldMap",
    "FieldDensities",
    "FocusDensities",
    "Method",
    "Orientations",
    "RayDensities",
    "ReferTo",
    "WaveDensities",
)

class FieldDensities(Enum):
    FieldDens_5 = 1
    FieldDens_10 = 2
    FieldDens_15 = 3
    FieldDens_20 = 4
    FieldDens_25 = 5
    FieldDens_30 = 6
    FieldDens_35 = 7
    FieldDens_40 = 8
    FieldDens_45 = 9
    FieldDens_50 = 10
    FieldDens_55 = 11
    FieldDens_60 = 12
    FieldDens_65 = 13
    FieldDens_70 = 14
    FieldDens_75 = 15
    FieldDens_80 = 16
    FieldDens_85 = 17
    FieldDens_90 = 18
    FieldDens_95 = 19
    FieldDens_100 = 20

class FocusDensities(Enum):
    FocusDens_5 = 1
    FocusDens_10 = 2
    FocusDens_15 = 3
    FocusDens_20 = 4
    FocusDens_25 = 5
    FocusDens_30 = 6
    FocusDens_35 = 7
    FocusDens_40 = 8
    FocusDens_45 = 9
    FocusDens_50 = 10
    FocusDens_55 = 11
    FocusDens_60 = 12
    FocusDens_65 = 13
    FocusDens_70 = 14
    FocusDens_75 = 15
    FocusDens_80 = 16
    FocusDens_85 = 17
    FocusDens_90 = 18
    FocusDens_95 = 19
    FocusDens_100 = 20

class Method(Enum):
    GaussQuad = 0
    RectArray = 1

class Orientations(Enum):
    Plus_Y = 0
    Minus_Y = 1
    Plus_X = 2
    Minus_X = 3

class RayDensities(Enum):
    RayDens_1 = 1
    RayDens_32x32 = 1
    RayDens_64x64 = 2
    RayDens_2 = 2
    RayDens_128x128 = 3
    RayDens_3 = 3
    RayDens_256x256 = 4
    RayDens_4 = 4
    RayDens_5 = 5
    RayDens_512x512 = 5
    RayDens_1024x1024 = 6
    RayDens_6 = 6
    RayDens_2048x2048 = 7
    RayDens_7 = 7
    RayDens_4096x4096 = 8
    RayDens_8 = 8
    RayDens_8192x8192 = 9
    RayDens_9 = 9
    RayDens_10 = 10
    RayDens_16384x16384 = 10
    RayDens_11 = 11
    RayDens_12 = 12
    RayDens_13 = 13
    RayDens_14 = 14
    RayDens_15 = 15
    RayDens_16 = 16
    RayDens_17 = 17
    RayDens_18 = 18
    RayDens_19 = 19
    RayDens_20 = 20

class ReferTo(Enum):
    ChiefRay = 0
    Centroid = 1

class WaveDensities(Enum):
    WaveDens_5 = 1
    WaveDens_10 = 2
    WaveDens_15 = 3
    WaveDens_20 = 4
    WaveDens_25 = 5
    WaveDens_30 = 6
    WaveDens_35 = 7
    WaveDens_40 = 8
    WaveDens_45 = 9
    WaveDens_50 = 10
    WaveDens_55 = 11
    WaveDens_60 = 12
    WaveDens_65 = 13
    WaveDens_70 = 14
    WaveDens_75 = 15
    WaveDens_80 = 16
    WaveDens_85 = 17
    WaveDens_90 = 18
    WaveDens_95 = 19
    WaveDens_100 = 20



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = ("LTEdgeSasmpling", "LTRaySampling", "NSCTraceOptions", "OPDMode", "RayStatus", "RaysType", "ZRDFormatType")

class LTEdgeSasmpling(Enum):
    S_1X_Low = 0
    S_4X = 1
    S_16X = 2
    S_64X = 3
    S_256X = 4

class LTRaySampling(Enum):
    S_1X_Low = 0
    S_4X = 1
    S_16X = 2
    S_64X = 3
    S_256X = 4
    S_1024X = 5

class NSCTraceOptions(Enum):
    None_ = 0
    UsePolarization = 1
    UseSplitting = 2
    UsePolarizationSplitting = 3
    UseScattering = 4
    UsePolarizationScattering = 5
    UseSplittingScattering = 6
    UsePolarizationSplittingScattering = 7

class OPDMode(Enum):
    None_ = 0
    Current = 1
    CurrentAndChief = 2

class RayStatus(Enum):
    Parent = 0
    Terminated = 1
    Reflected = 2
    Transmitted = 4
    Scattered = 8
    Diffracted = 16
    GhostedFrom = 32
    DiffractedFrom = 64
    ScatteredFrom = 128
    RayError = 256
    BulkScattered = 512
    WaveShifted = 1024
    OrdinaryRay = 65536
    ExtraordinaryRay = 131072
    WaveShiftPL = 262144

class RaysType(Enum):
    Real = 0
    Paraxial = 1

class ZRDFormatType(Enum):
    UncompressedFullData = 0
    CompressedBasicData = 1
    CompressedFullData = 2



"""Zemax OpticStudio analyses from the Wavefront category."""

from __future__ import annotations

import os
import re
from tempfile import mkstemp

import numpy as np
import pandas as pd

from zospy import utils
from zospy.analyses.base import AnalysisResult, AttrDict, OnComplete, new_analysis
from zospy.api import constants
from zospy.zpcore import OpticStudioSystem


def _structure_zernike_standard_coefficients_result(line_list: list[str]) -> tuple[pd.DataFrame, pd.DataFrame]:
    """Structures the result of a zernike standard coefficients analysis.

    Parameters
    ----------
    line_list: list of str
        The line list obtained by reading in the results

    Returns
    -------
    (pd.DataFrame, pd.DataFrame)
        Two dataframes, respectively the general results and the coefficients
    """
    zlinepat = re.compile(r"^Z\s+\d+")
    valuepat_start = re.compile(r"^((-)?\d+\.\d+)")
    valuepat_any = re.compile(r"((-)?\d+(\.\d+)?)")

    zernike_lines = [line for line in line_list if (zlinepat.search(line) is not None)]

    general_lines = [line for line in line_list if line not in zernike_lines]
    general_arr = [" ".join(line.split()) for line in general_lines if ":" in line]
    zernike_arr = np.array([line.replace(" ", "").split() for line in zernike_lines])
    general_data = pd.DataFrame(columns=["Value", "Unit"])
    for line in general_arr:
        spl = line.split(":", 1)
        if len(spl) == 1:
            ind = "".join([item.title() for item in spl[0].split()])
            general_data.loc[ind] = ["", ""]
        else:
            ind = "".join([item.title() for item in spl[0].split()])
            nvals = len(valuepat_any.findall(spl[1].replace(",", ".")))
            dat = spl[1].strip().split(maxsplit=nvals)
            if len(dat) == 0:
                val = ""
                unit = ""
                general_data.loc[ind] = [val, unit]
            elif len(dat) == 1:
                if valuepat_start.search(dat[0].replace(",", ".")):  # value is number
                    val = float(dat[0].replace(",", "."))
                else:
                    val = dat[0]
                unit = ""
                general_data.loc[ind] = [val, unit]
            elif len(dat) == 2:
                if valuepat_start.search(dat[0].replace(",", ".")):  # value is number
                    val = float(dat[0].replace(",", "."))
                else:
                    val = dat[0]
                unit = dat[1]
                general_data.loc[ind] = [val, unit]
            else:
                for ii in range(len(dat) - 1):
                    if valuepat_start.search(dat[ii].replace(",", ".")):  # value is number
                        val = float(valuepat_start.search(dat[ii].replace(",", ".")).group())
                    else:
                        val = dat[ii]
                    unit = dat[-1]
                    general_data.loc["{}_{}".format(ind, ii)] = [val, unit]
    zernike_data = pd.DataFrame(index=zernike_arr[:, 0].copy(), columns=["Value", "Unit", "Function"])
    zernike_data.loc[zernike_arr[:, 0], "Value"] = list(
        map(lambda s: float(s.replace(",", ".")), zernike_arr[:, 1].copy())
    )
    zernike_data.loc[zernike_arr[:, 0], "Unit"] = "waves"
    zernike_data.loc[zernike_arr[:, 0], "Function"] = zernike_arr[:, 3].copy()

    return general_data, zernike_data


def zernike_standard_coefficients(
    oss: OpticStudioSystem,
    sampling: str = "64x64",
    maximum_term: int = 37,
    wavelength: str | int = 1,
    field: str | int = 1,
    reference_opd_to_vertex: bool = False,
    surface: str | int = "Image",
    sx: float = 0.0,
    sy: float = 0.0,
    sr: float = 0.0,
    oncomplete: OnComplete | str = OnComplete.Close,
    txtoutfile: str = None,
) -> AnalysisResult:
    """Wrapper around the OpticStudio Zernike Standard Coefficient Analysis.

    Parameters
    ----------
    oss: zospy.core.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance. Should be sequential.
    sampling: str | int
        The sampling, either string (e.g. '64x64') or int. The integer will be treated as a ZOSAPI Constants integer.
    maximum_term: int
        The maximum Zernike term, defaults to 37.
    wavelength: int
        The wavelength number that is to be used. Defaults to 1 (the first wavelength).
    field:
        The field that is to be analyzed. Defaults to 1.
    reference_opd_to_vertex: bool
        If the OPD should be referenced to vertex. Defaults to False.
    surface: str | int
        The surface that is to be analyzed. Either 'Image', 'Object' or an integer. Defaults to 'Image'.
    sx: float
        The sx.
    sy: float
        The sy.
    sr: float
        The sr.
    oncomplete: OnComplete | str
        Defines behaviour upon completion of the analysis. Should be one of ['Close', 'Release', 'Sustain']. If 'Close',
        the analysis will be closed after completion. If 'Release', the analysis will remain open in OpticStudio, but
        the link with python will be destroyed. If 'Sustain' the analysis will be kept open in OpticStudio and the link
        with python will be sustained. To enable interaction when oncomplete == 'Sustain', the OpticStudio Analysis
        instance will be available in the returned AnalysisResult through AnalysisResult.Analysis. Defaults to 'Close'.
    txtoutfile: str | None
        The textfile to which the OpticStudio output is saved. If None, a temporary file will be created and deleted. If
        string, it should be a full system path with '.txt' as extension, after which the file will be saved in that
        location. Defaults to None.

    Returns
    -------
    AnalysisResult
        A ZernikeStandardCoefficients analysis result. Next to the standard data, the raw text return obtained from the
        analysis will be present under 'RawTextData' and the txtoutfile under 'TxtOutFile'.
    """
    analysis_type = constants.Analysis.AnalysisIDM.ZernikeStandardCoefficients

    if txtoutfile is None:
        fd, txtoutfile = mkstemp(suffix=".txt", prefix="zospy_")
        os.close(fd)
        cleantxt = True
    else:
        if not txtoutfile.endswith(".txt"):
            raise ValueError('txtfile should end with ".txt"')
        cleantxt = False

    # Create analysis
    analysis = new_analysis(oss, analysis_type)

    # Apply settings
    analysis.Settings.SampleSize = getattr(constants.Analysis.SampleSizes, utils.zputils.standardize_sampling(sampling))
    analysis.Settings.MaximumNumberOfTerms = maximum_term
    analysis.set_wavelength(wavelength)
    analysis.set_field(field)
    analysis.Settings.ReferenceOBDToVertex = reference_opd_to_vertex  # ToDo: Monitor name with zemax updates
    analysis.set_surface(surface)
    analysis.Settings.Sx = sx
    analysis.Settings.Sy = sy
    analysis.Settings.Sr = sr

    # Calculate
    analysis.ApplyAndWaitForCompletion()

    # Get results
    analysis.Results.GetTextFile(txtoutfile)
    line_list = [line for line in open(txtoutfile, "r", encoding=oss._ZOS.get_txtfile_encoding())]

    general_data, zernike_data = _structure_zernike_standard_coefficients_result(line_list)
    data = AttrDict(GeneralData=general_data, Coefficients=zernike_data)

    # Get headerdata, metadata and messages
    headerdata = analysis.get_header_data()
    metadata = analysis.get_metadata()
    messages = analysis.get_messages()

    # Manually create settings as they cannot be accessed
    settings = pd.Series(name="Settings", dtype=object)

    settings.loc["SampleSize"] = str(analysis.Settings.SampleSize)
    settings.loc["MaximumNumberOfTerms"] = analysis.Settings.MaximumNumberOfTerms
    settings.loc["Wavelength"] = analysis.Settings.Wavelength.GetWavelengthNumber()  # Todo Evaluate with 'all'
    settings.loc["Field"] = analysis.Settings.Field.GetFieldNumber()  # Todo Evaluate with 'all'
    settings.loc["ReferenceOBDToVertex"] = analysis.Settings.ReferenceOBDToVertex
    settings.loc["Wavelength"] = analysis.Settings.Wavelength.GetWavelengthNumber()  # Todo Evaluate with 'all'
    settings.loc["Field"] = analysis.Settings.Field.GetFieldNumber()  # Todo Evaluate with 'all'
    settings.loc["ReferenceOBDToVertex"] = analysis.Settings.Surface.GetSurfaceNumber()  # Todo Evaluate with 'all'
    settings.loc["Sx"] = sx
    settings.loc["Sy"] = sy
    settings.loc["Sr"] = sr

    # Create output
    result = AnalysisResult(
        analysistype=str(analysis_type),
        data=data,
        settings=settings,
        metadata=metadata,
        headerdata=headerdata,
        messages=messages,
        RawTextData=line_list,
        TxtOutFile=txtoutfile,
    )  # Set additional params

    # cleanup
    if cleantxt:
        os.remove(txtoutfile)

    return analysis.complete(oncomplete, result)



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Analysis import SampleSizes
from zospy.api._ZOSAPI.Analysis.Settings import (
    IAS_,
    IAS_Field,
    IAS_Surface,
    IAS_Wavelength,
)

__all__ = (
    "DisplayAsTypes",
    "Distortions",
    "FFA_AberrationTypes",
    "FFA_DecompositionTypes",
    "FFA_DisplayTypes",
    "FFA_FieldShapes",
    "FFA_ShowAsTypes",
    "FieldScanDirections",
    "IAS_FieldCurvatureAndDistortion",
    "IAS_FocalShiftDiagram",
    "IAS_FullFieldAberration",
    "IAS_GridDistortion",
    "IAS_LateralColor",
    "IAS_LongitudinalAberration",
    "IAS_RayTrace",
    "IAS_SeidelCoefficients",
    "IAS_SeidelDiagram",
    "IAS_ZernikeAnnularCoefficients",
    "IAS_ZernikeCoefficientsVsField",
    "IAS_ZernikeFringeCoefficients",
    "IAS_ZernikeStandardCoefficients",
    "RayTraceType",
    "ZernikeCoefficientTypes",
)

class DisplayAsTypes:
    Percent = 0
    Absolute = 1

class Distortions:
    F_TanTheta = 0
    F_Theta = 1
    Cal_F_Theta = 2
    Cal_F_TanTheta = 3
    SMIA_TV = 4

class FFA_AberrationTypes:
    Defocus = 0
    PrimaryAstigmatism = 1
    PrimaryComa = 2
    SpecifiedTerm = 3

class FFA_DecompositionTypes:
    ZernikeTerms = 0

class FFA_DisplayTypes:
    Absolute = 0
    Relative = 1
    Average = 2

class FFA_FieldShapes:
    Rectangular = 0
    Elliptical = 1

class FFA_ShowAsTypes:
    GreyScale = 0
    GreyScaleInverted = 1
    FalseColor = 2
    FalseColorInverted = 3
    Icons = 4

class FieldScanDirections:
    Plus_Y = 0
    Plus_X = 1
    Minus_Y = 2
    Minus_X = 3

class IAS_FieldCurvatureAndDistortion(IAS_):
    @property
    def DisplayAs(self) -> DisplayAsTypes: ...
    @property
    def Distortion(self) -> Distortions: ...
    @property
    def FieldAspectRatio(self) -> float: ...
    @property
    def IgnoreVignette(self) -> bool: ...
    @property
    def MaximumCurvature(self) -> float: ...
    @property
    def MaximumDistortion(self) -> float: ...
    @property
    def ReferenceField(self) -> IAS_Field: ...
    @property
    def ScanType(self) -> FieldScanDirections: ...
    @property
    def UseDashes(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @DisplayAs.setter
    def DisplayAs(self, value: DisplayAsTypes) -> None: ...
    @Distortion.setter
    def Distortion(self, value: Distortions) -> None: ...
    @FieldAspectRatio.setter
    def FieldAspectRatio(self, value: float) -> None: ...
    @IgnoreVignette.setter
    def IgnoreVignette(self, value: bool) -> None: ...
    @MaximumCurvature.setter
    def MaximumCurvature(self, value: float) -> None: ...
    @MaximumDistortion.setter
    def MaximumDistortion(self, value: float) -> None: ...
    @ScanType.setter
    def ScanType(self, value: FieldScanDirections) -> None: ...
    @UseDashes.setter
    def UseDashes(self, value: bool) -> None: ...

class IAS_FocalShiftDiagram(IAS_):
    @property
    def MaximumShift(self) -> float: ...
    @property
    def PupilZone(self) -> float: ...
    @MaximumShift.setter
    def MaximumShift(self, value: float) -> None: ...
    @PupilZone.setter
    def PupilZone(self, value: float) -> None: ...

class IAS_FullFieldAberration(IAS_):
    @property
    def AberrationNumber(self) -> int: ...
    @property
    def AberrationType(self) -> FFA_AberrationTypes: ...
    @property
    def Decomposition(self) -> FFA_DecompositionTypes: ...
    @property
    def Display(self) -> FFA_DisplayTypes: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def FieldShape(self) -> FFA_FieldShapes: ...
    @property
    def MaximumTerm(self) -> int: ...
    @property
    def PupilSampling(self) -> SampleSizes: ...
    @property
    def ShowAs(self) -> FFA_ShowAsTypes: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @property
    def XFieldSampling(self) -> int: ...
    @property
    def XFieldWidth(self) -> float: ...
    @property
    def YFieldSampling(self) -> int: ...
    @property
    def YFieldWidth(self) -> float: ...
    @Decomposition.setter
    def Decomposition(self, value: FFA_DecompositionTypes) -> None: ...
    @Display.setter
    def Display(self, value: FFA_DisplayTypes) -> None: ...
    @FieldShape.setter
    def FieldShape(self, value: FFA_FieldShapes) -> None: ...
    @MaximumTerm.setter
    def MaximumTerm(self, value: int) -> None: ...
    @PupilSampling.setter
    def PupilSampling(self, value: SampleSizes) -> None: ...
    @ShowAs.setter
    def ShowAs(self, value: FFA_ShowAsTypes) -> None: ...
    @XFieldSampling.setter
    def XFieldSampling(self, value: int) -> None: ...
    @XFieldWidth.setter
    def XFieldWidth(self, value: float) -> None: ...
    @YFieldSampling.setter
    def YFieldSampling(self, value: int) -> None: ...
    @YFieldWidth.setter
    def YFieldWidth(self, value: float) -> None: ...
    def SetAberrationByNumber(self, aberrationNumber: int) -> bool: ...
    def SetAberrationByType(self, type: FFA_AberrationTypes) -> bool: ...

class IAS_GridDistortion(IAS_):
    @property
    def Aspect(self) -> float: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def FieldWidth(self) -> float: ...
    @property
    def GridNumber(self) -> int: ...
    @property
    def Method(self) -> int: ...
    @property
    def RotateText(self) -> int: ...
    @property
    def ScaleFactor(self) -> float: ...
    @property
    def SymmetricMagnification(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @Aspect.setter
    def Aspect(self, value: float) -> None: ...
    @FieldWidth.setter
    def FieldWidth(self, value: float) -> None: ...
    @GridNumber.setter
    def GridNumber(self, value: int) -> None: ...
    @Method.setter
    def Method(self, value: int) -> None: ...
    @RotateText.setter
    def RotateText(self, value: int) -> None: ...
    @ScaleFactor.setter
    def ScaleFactor(self, value: float) -> None: ...
    @SymmetricMagnification.setter
    def SymmetricMagnification(self, value: bool) -> None: ...

class IAS_LateralColor(IAS_):
    @property
    def AllWavelengths(self) -> bool: ...
    @property
    def PlotScale(self) -> float: ...
    @property
    def ShowAiryDisk(self) -> bool: ...
    @property
    def UseRealRays(self) -> bool: ...
    @AllWavelengths.setter
    def AllWavelengths(self, value: bool) -> None: ...
    @PlotScale.setter
    def PlotScale(self, value: float) -> None: ...
    @ShowAiryDisk.setter
    def ShowAiryDisk(self, value: bool) -> None: ...
    @UseRealRays.setter
    def UseRealRays(self, value: bool) -> None: ...

class IAS_LongitudinalAberration(IAS_):
    @property
    def PlotScale(self) -> float: ...
    @property
    def UseDashes(self) -> bool: ...
    @PlotScale.setter
    def PlotScale(self, value: float) -> None: ...
    @UseDashes.setter
    def UseDashes(self, value: bool) -> None: ...

class IAS_RayTrace(IAS_):
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def Hx(self) -> float: ...
    @property
    def Hy(self) -> float: ...
    @property
    def Px(self) -> float: ...
    @property
    def Py(self) -> float: ...
    @property
    def Type(self) -> RayTraceType: ...
    @property
    def UseGlobal(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @Hx.setter
    def Hx(self, value: float) -> None: ...
    @Hy.setter
    def Hy(self, value: float) -> None: ...
    @Px.setter
    def Px(self, value: float) -> None: ...
    @Py.setter
    def Py(self, value: float) -> None: ...
    @Type.setter
    def Type(self, value: RayTraceType) -> None: ...
    @UseGlobal.setter
    def UseGlobal(self, value: bool) -> None: ...
    def UseArbitraryRay(self) -> None: ...

class IAS_SeidelCoefficients(IAS_):
    @property
    def Wavelength(self) -> IAS_Wavelength: ...

class IAS_SeidelDiagram(IAS_):
    @property
    def IgnoreChromatic(self) -> bool: ...
    @property
    def IgnoreDistortion(self) -> bool: ...
    @property
    def PlotScale(self) -> float: ...
    @property
    def SuppressFrame(self) -> bool: ...
    @IgnoreChromatic.setter
    def IgnoreChromatic(self, value: bool) -> None: ...
    @IgnoreDistortion.setter
    def IgnoreDistortion(self, value: bool) -> None: ...
    @PlotScale.setter
    def PlotScale(self, value: float) -> None: ...
    @SuppressFrame.setter
    def SuppressFrame(self, value: bool) -> None: ...

class IAS_ZernikeAnnularCoefficients(IAS_):
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def MaximumNumberOfTerms(self) -> int: ...
    @property
    def Obscuration(self) -> float: ...
    @property
    def ReferenceOBDToVertex(self) -> bool: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def Sr(self) -> float: ...
    @property
    def Surface(self) -> IAS_Surface: ...
    @property
    def Sx(self) -> float: ...
    @property
    def Sy(self) -> float: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @MaximumNumberOfTerms.setter
    def MaximumNumberOfTerms(self, value: int) -> None: ...
    @Obscuration.setter
    def Obscuration(self, value: float) -> None: ...
    @ReferenceOBDToVertex.setter
    def ReferenceOBDToVertex(self, value: bool) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @Sr.setter
    def Sr(self, value: float) -> None: ...
    @Sx.setter
    def Sx(self, value: float) -> None: ...
    @Sy.setter
    def Sy(self, value: float) -> None: ...

class IAS_ZernikeCoefficientsVsField(IAS_):
    @property
    def Coefficients(self) -> str: ...
    @property
    def FieldDensity(self) -> int: ...
    @property
    def FieldScanDirection(self) -> FieldScanDirections: ...
    @property
    def ObscurationFactor(self) -> float: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def ScaleMaximum(self) -> float: ...
    @property
    def ScaleMinimum(self) -> float: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @property
    def ZernikeCoefficientType(self) -> ZernikeCoefficientTypes: ...
    @Coefficients.setter
    def Coefficients(self, value: str) -> None: ...
    @FieldDensity.setter
    def FieldDensity(self, value: int) -> None: ...
    @FieldScanDirection.setter
    def FieldScanDirection(self, value: FieldScanDirections) -> None: ...
    @ObscurationFactor.setter
    def ObscurationFactor(self, value: float) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @ScaleMaximum.setter
    def ScaleMaximum(self, value: float) -> None: ...
    @ScaleMinimum.setter
    def ScaleMinimum(self, value: float) -> None: ...
    @ZernikeCoefficientType.setter
    def ZernikeCoefficientType(self, value: ZernikeCoefficientTypes) -> None: ...

class IAS_ZernikeFringeCoefficients(IAS_):
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def MaximumNumberOfTerms(self) -> int: ...
    @property
    def ReferenceOBDToVertex(self) -> bool: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def Sr(self) -> float: ...
    @property
    def Surface(self) -> IAS_Surface: ...
    @property
    def Sx(self) -> float: ...
    @property
    def Sy(self) -> float: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @MaximumNumberOfTerms.setter
    def MaximumNumberOfTerms(self, value: int) -> None: ...
    @ReferenceOBDToVertex.setter
    def ReferenceOBDToVertex(self, value: bool) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @Sr.setter
    def Sr(self, value: float) -> None: ...
    @Sx.setter
    def Sx(self, value: float) -> None: ...
    @Sy.setter
    def Sy(self, value: float) -> None: ...

class IAS_ZernikeStandardCoefficients(IAS_):
    @property
    def Epsilon(self) -> float: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def MaximumNumberOfTerms(self) -> int: ...
    @property
    def ReferenceOBDToVertex(self) -> bool: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def Sr(self) -> float: ...
    @property
    def Surface(self) -> IAS_Surface: ...
    @property
    def Sx(self) -> float: ...
    @property
    def Sy(self) -> float: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @Epsilon.setter
    def Epsilon(self, value: float) -> None: ...
    @MaximumNumberOfTerms.setter
    def MaximumNumberOfTerms(self, value: int) -> None: ...
    @ReferenceOBDToVertex.setter
    def ReferenceOBDToVertex(self, value: bool) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @Sr.setter
    def Sr(self, value: float) -> None: ...
    @Sx.setter
    def Sx(self, value: float) -> None: ...
    @Sy.setter
    def Sy(self, value: float) -> None: ...

class RayTraceType:
    DirectionCosines = 0
    TangentAngle = 1
    YmUmYcUc = 2

class ZernikeCoefficientTypes:
    Fringe = 0
    Standard = 1
    Annular = 2
